#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================

"""subcommands
    * pydorkをコマンドとして動作させる際の処理を記載しているモジュール
"""


import sys
import threading
import json
import os
import pathlib

from typing import List
from argparse import Namespace
from jinja2 import Template

from .engine import SearchEngine, ENGINES
from .common import Color
from .common import Message


# サブコマンドの動作集約用関数
def run_subcommand(subcommand, args):
    """run_subcommand

    Args:
        subcommand (str): 使用するサブコマンド([search, suggest]).
        args (Namespace): argparseで取得した引数(Namespace).
    """

    # template file用の変数セット(dict)
    template_variable = {}

    # query及びfileがともに指定なしの場合、エラーにして返す
    if args.query == "" and args.file == "" and args.template_file == "":
        print("Error: クエリもしくはファイルを指定してください.", file=sys.stderr)
        return

    # args.fileのチェック
    if args.file != "":
        if not os.path.exists(args.file):
            print("Error: ファイルが存在しません.", file=sys.stderr)
            return

    # args.template_fileのチェック
    if args.template_file != "":
        if not os.path.exists(args.template_file):
            print("Error: ファイルが存在しません.", file=sys.stderr)
            return

        if args.template_variable == "":
            print("Error: テンプレート変数が指定されていません.", file=sys.stderr)
            return

        try:
            template_variable = json.loads(args.template_variable)
        except Exception:
            print("Error: テンプレート変数の形式がまちがっています.", file=sys.stderr)
            return

    # 各サブコマンドのチェック
    target = None
    search_mode = ''
    if subcommand == 'search':
        # チェック処理
        if ((args.start is None and args.end is not None) or (args.start is not None and args.end is None)):
            print(
                Color.GRAY + "期間を指定する場合は--start, --endの両方を指定してください" + Color.END,
                file=sys.stderr
            )
            return
        target = run_search
        search_mode = 'text'

    elif subcommand == 'image':
        target = run_search
        search_mode = 'image'

    elif subcommand == 'suggest':
        target = run_suggest

    # create query_list
    query_list = generate_query_list(args)

    # append query in template file
    if args.template_file != "":
        # template fileのfullpathを取得
        template_file = pathlib.Path(args.template_file).expanduser()

        # args.template_variableをjsonとして読み込む.

        with open(template_file) as f:
            template_data = f.read()

            # template fileから値を取得
            tmpl = Template(template_data)

            # 設定情報を取得
            tmpl_params = template_variable

            # レンダリング処理を実行
            rendered_query_strings = tmpl.render(tmpl_params)

            # templateを1行ずつqueryに追加
            template_file_querys = [s.strip()
                                    for s in rendered_query_strings.splitlines()]
            query_list.extend(template_file_querys)

    # engine_listへ、選択されているsearch engineを入れていく
    engine_list = []
    for search_type in args.search_type:
        # if all
        if search_type == 'all':
            for engine in ENGINES:
                engine_list.append(engine)
            continue

        # if in searchengine
        if search_type in ENGINES:
            engine_list.append(search_type)
            continue

    # engine_listから、重複したリストを削除
    engine_list = list(set(engine_list))

    tasks = []
    thread_result = dict()
    lock = threading.Lock()
    for engine in engine_list:
        task = threading.Thread(
            target=target, args=(engine, query_list, args, thread_result, True, lock, search_mode))
        tasks.append(task)

    for task in tasks:
        task.start()

    for task in tasks:
        task.join()

    # json出力が有効だった場合、json形式で出力
    if args.json:
        print(json.dumps(thread_result, ensure_ascii=False, indent=2))


# SearchEngineのオプション設定用関数
def set_se_options(se: SearchEngine, args: Namespace):
    """set_se_options

    Args:
        se (SearchEngine): argsの情報を元に、オプションを設定するSearchEngine.
        args (Namespace): argparseで取得した引数(Namespace).

    Returns:
        SearchEngine: オプションを設定したSearchEngine.
    """

    # set debug flag
    if 'debug' in args:
        se.set_is_debug(args.debug)

    # set ssl verify
    if 'insecure' in args:
        se.set_ignore_ssl(args.insecure)

    # set is_command flag
    se.set_is_command(True)

    # set disable headless
    if 'disable_headless' in args:
        se.set_disable_headless(args.disable_headless)

    # proxy
    if args.proxy != '':
        se.set_proxy(args.proxy)

    # Selenium
    if args.selenium:
        # set default endpoint
        endpoint = None

        # if set browser-endpoint
        if args.browser_endpoint != "":
            endpoint = args.browser_endpoint

        # set selenium
        se.set_selenium(endpoint, args.browser)

    # Splush
    if args.splash:
        # set default endpoint
        endpoint = 'localhost:8050'

        # if set browser-endpoint
        if args.browser_endpoint != "":
            endpoint = args.browser_endpoint

        # set splash
        se.set_splash(endpoint)

    # useragent
    se.set_user_agent()

    # lang/country code
    se.set_lang(args.lang, args.country)

    # set cookie driver(last set)
    se.set_cookie_files(args.cookies)

    # set cookie file delete
    se.set_cookie_files_delete(args.delete_cookies)

    return se


# 検索結果を出力する
def print_search_result(result, args: Namespace, message: Message):
    """print_search_result


    Args:
        result : SearchEngine.searchのresult.
        args (Namespace): argparseで取得した引数(Namespace).
        message (common.Message): 出力用Class.
    """

    # 区切り文字を指定
    sep = ': '
    if args.nullchar:
        sep = '\0'

    # title出力を行うか確認
    title_mode = False
    if 'title' in args:
        title_mode = args.title

    # pageurl出力を行うか確認
    pagelink_mode = False
    if 'pagelink' in args:
        pagelink_mode = args.pagelink

    for d in result:
        data = []
        link = d['link']

        # 出力dataにlinkを追加
        data.insert(0, link)

        # pageurlの有無を確認
        if 'pagelink' in d and pagelink_mode:
            pagelink = d['pagelink']

            # pagelinkの色指定
            if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
                pagelink = Color.GRAY + Color.UNDERLINE + pagelink + Color.END

            data.insert(0, pagelink)

        # titleの有無を確認
        if 'title' in d and title_mode:
            title = d['title']

            # titleの色指定
            if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
                title = Color.GRAY + title + Color.END

            data.insert(0, title)

        message.print_line(*data, separator=sep)


# generate
def generate_query_list(args: Namespace):
    """generate_query_list

    """
    # create query_list
    query_list: List[str] = list()

    # append query
    if args.query != "":
        query_list.append(args.query)

    # append query in file
    if args.file != "":
        # fileのfull pathを取得
        file = pathlib.Path(args.file).expanduser()

        # ファイルを開いて1行ずつqueryに追加する
        with open(file) as f:
            file_querys = [s.strip() for s in f.readlines()]
            query_list.extend(file_querys)

    return query_list


# 検索
def run_search(engine: str, query_list: list, args, thread_result: dict, cmd=False, lock=None, mode='text'):
    """search

    Args:
        engine (str): 使用する検索エンジン(.engine.ENGINES).
        query_list(list): 検索クエリのリスト.
        args (Namespace): argparseで取得した引数(Namespace).
        thread_result(dict): 結果を1箇所に集約するためのresult dict. json出力するときのみ使用.
        cmd (bool, optional): commandで実行しているか否か. Defaults to False.
        lock (threading.Lock): threadingのマルチスレッドで使用するLock.現在は未使用. Defaults to None.
        type (str, optional): 検索タイプ. `text` or `image`.
    """

    # start SearchEngine class
    se = SearchEngine()

    # Set Engine
    se.set(engine)

    # Set SearchEngine options
    se = set_se_options(se, args)

    # Set lock
    se.set_lock(lock)

    # Set color
    if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
        se.set_is_color(True)

    # 検索タイプを設定(テキスト or 画像)
    search_type = mode

    # 区切り文字を指定
    sep = ': '
    if args.nullchar:
        sep = '\0'

    # json出力時の変数を宣言
    all_result_json = list()

    # query_listの内容を順番に処理
    for query in query_list:
        # 検索を実行
        result = se.search(
            query, search_type=search_type,
            maximum=args.num
        )

        # debug
        se.ENGINE.MESSAGE.print_text(
            json.dumps(result),
            separator=sep,
            header=se.ENGINE.MESSAGE.HEADER + ': ' +
            Color.GRAY + '[DEBUG]: [Result]' + Color.END,
            mode="debug",
        )

        if args.json:
            # all_result_jsonへ組み込むためのjson方式へ加工.
            append_result = {
                'query': query,
                'result': result
            }
            all_result_json.append(append_result)

        else:
            print_search_result(result, args, se.ENGINE.MESSAGE)

    if args.json:
        thread_result[engine] = all_result_json


# サジェスト
def run_suggest(engine: str, query_list: list, args: Namespace, thread_result: dict, cmd=False, lock=None, mode=''):
    """suggest

    Args:
        engine (str): 使用する検索エンジン(.engine.ENGINES).
        query_list(list): 検索クエリのリスト.
        args (Namespace): argparseで取得した引数(Namespace).
        thread_result(dict): 結果を1箇所に集約するためのresult dict. json出力するときのみ使用.
        cmd (bool, optional): commandで実行しているか否か. Defaults to False.
        lock (threading.Lock): threadingのマルチスレッドで使用するLock.現在は未使用. Defaults to None.
        mode (str, optional): マルチスレッドでsearchある程度共用で使えるようにするための引数. 利用していない. Defaults to ''.
    """

    # start search engine class
    se = SearchEngine()

    # Set Engine
    se.set(engine)

    # Set Message()
    msg = Message()
    msg.set_engine(se.ENGINE.NAME, se.ENGINE.COLOR)
    if 'debug' in args:
        msg.set_is_debug(args.debug)
    msg.set_is_command(True)

    # set msg to se
    se.ENGINE.set_messages(msg)

    # Set SearchEngine options
    se = set_se_options(se, args)

    # Set lock
    se.set_lock(lock)

    # Header
    header = '[${ENGINE_NAME}Suggest]'
    if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
        sc = Color(se.ENGINE.COLOR)
        header = sc.out(header)
    se.ENGINE.MESSAGE.set_header(header)

    # json出力時の変数を宣言
    all_result_json = list()

    # Suggestを取得
    for query in query_list:
        result = se.suggest(
            query,
            jap=args.jap,
            alph=args.alph,
            num=args.num,
        )

        for words in result.values():
            if args.json:
                append_result = {
                    'query': query,
                    'result': words
                }
                all_result_json.append(append_result)

            else:
                for w in words:
                    se.ENGINE.MESSAGE.print_line(w, separator=": ")

    if args.json:
        thread_result[engine] = all_result_json



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""common
    * 共通系や雑多な処理を詰め合わせたバルクモジュール.
"""

import sys
import datetime

from string import Template


# コンソール出力時に色付をするためのClass
class Color:
    """Color

    コンソール出力時に色付をするための文字列を変数にして保持しているClass.

    Examples:
        c = Color()
        c.set(c.BLUE)
        print(c.out('hogehoge'))
    """
    # color_code
    BLACK = '\033[30m'
    RED = '\033[31m'
    GREEN = '\033[32m'
    YELLOW = '\033[33m'
    BLUE = '\033[34m'
    PURPLE = '\033[35m'
    CYAN = '\033[36m'
    WHITE = '\033[37m'
    GRAY = '\033[1;30m'

    # 文字効果
    BOLD = '\038[1m'
    ITALIC = '\038[3m'
    UNDERLINE = '\033[4m'
    INVISIBLE = '\033[08m'
    REVERCE = '\033[07m'

    # 効果を終了
    END = '\033[0m'

    def __init__(self, color_code: str):
        """[summary]

        Args:
            color_code (str): 使用するカラーコード

        """
        self.COLOR_CODE = color_code

    def out(self, text: str, is_bold=False, is_underline=False, is_reverse=False, is_italic=False):
        # textを囲む
        text = self.COLOR_CODE + text + self.END

        # is_boldが有効な場合、太字にする
        if is_bold:
            text = self.BOLD + text + self.END

        # is_underlineが有効な場合、下線を入れる
        if is_underline:
            text = self.UNDERLINE + text + self.END

        # is_reverseが有効な場合、色の反転をする
        if is_reverse:
            text = self.REVERCE + text + self.END

        # is_italicが有効な場合、Italicにする
        if is_italic:
            text = self.ITALIC + text + self.END

        return text


# Message関連の制御用Class
class Message:
    """Message

    メッセージの出力を簡易化するためのClass.

    Examples:

    """

    def __init__(self):
        # command flag
        self.IS_COMMAND = False

        # debug flag
        self.IS_DEBUG = False

        # timestamp flag
        self.IS_TIMESTAMP = False

        # engine data
        self.ENGINE_COLOR = Color('')
        self.ENGINE_NAME = ''
        self.ENGINE = ''

        # header
        self.HEADER = ''

    def set_is_command(self, is_command: bool):
        self.IS_COMMAND = is_command

    def set_is_debug(self, is_debug: bool):
        self.IS_DEBUG = is_debug

    def set_engine(self, engine: str, color: str):
        self.ENGINE_COLOR = Color(color)
        self.ENGINE_NAME = engine
        self.ENGINE = self.ENGINE_COLOR.out(engine)

    def set_header(self, text):
        self.HEADER = text

    def replace(self, text):
        """replace

        テンプレートテキストの変数をself変数や時刻に置換して返す

        Args:
            text (str): 置換処理をするテンプレート用テキスト
        """

        # 現在時刻を取得
        dt_now = datetime.datetime.now()

        # 置換用のdictを生成
        data = {
            # 時刻情報
            'YEAR': dt_now.year,
            'MONTH': dt_now.month,
            'DAY': dt_now.day,
            'HOUR': dt_now.hour,
            'MINUTE': dt_now.minute,
            'SECOND': dt_now.second,

            # 検索エンジン(color)
            'ENGINE': self.ENGINE,  # 色付き
            'ENGINE_NAME': self.ENGINE_NAME,  # 色なし
        }

        # テンプレートを作成
        template = Template(text)

        # 置換処理を実行
        result = template.safe_substitute(data)

        return result

    def print_line(self, *text, use_header=True, separator=' ', file=sys.stdout, header=None):
        """print_line

        メッセージを出力する(行)

        Args:
            text: メッセージとして出力するテキスト行
            use_header: `header`で指定しているヘッダーを行頭に表示するかどうか
            separator: printする際に使用する区切り文字
            file: 出力先のファイル(デフォルトはstdout)
            header: ヘッダーとして使用する文字列を指定
        """
        # headerの生成
        if header is None:
            header = self.HEADER

        header = self.replace(header)

        # テキストを出力
        if use_header:
            print(header, *text, sep=separator, file=file)
        else:
            print(*text, sep=separator, file=file)

    def print_text(self, text, mode='message', use_header=True, separator=' ', file=sys.stdout, header=None):
        """print_line

        メッセージを出力する(テキスト)

        Args:
            text: メッセージとして出力するテキスト
            mode: メッセージの出力モード(`message`, `error`, `warn`, `info`, `debug`)
            use_header: `header`で指定しているヘッダーを行頭に表示するかどうか
            separator: printする際に使用する区切り文字
            file: 出力先のファイル(デフォルトはstdout)
            header: ヘッダーとして使用する文字列を指定
        """
        # is_commandが有効のときのみ出力させる
        if not self.IS_COMMAND:
            return

        # debug, infoのときは、self.is_debugが有効のときのみ出力
        if mode in ('info', 'debug'):
            # self.is_debugでない場合は出力しない
            if not self.IS_DEBUG:
                return

        # 出力テキストの生成
        text = self.replace(text)

        # case
        text_color: Color = Color(Color.END)
        if mode == 'message':  # modeが `message` のとき
            text_color = Color(Color.WHITE)

        elif mode == 'error':
            text_color = Color(Color.RED)
            file = sys.stderr

        elif mode == 'warn':
            text_color = Color(Color.YELLOW)
            file = sys.stderr

        elif mode == 'info':
            text_color = Color(Color.GREEN)
            file = sys.stderr

        elif mode == 'debug':
            text_color = Color(Color.GRAY)
            file = sys.stderr

        # default headerの定義
        if mode in ('info', 'debug'):
            if header is None:
                header = self.HEADER

            header = Color.REVERCE + \
                self.replace(header) + Color.END

        # TODO: 正規表現で、付きの箇所を抜き出すような処理を追加で入れる

        # テキストの出力
        for line in text.splitlines():
            self.print_line(text_color.out(line),
                            separator=separator, use_header=use_header, file=file, header=header)

        return


# 渡されたリスト内のdictに`num`を追加する関数
def set_counter(links: list):
    """set_counter

    links(list)の要素に`num`キーを追加し、連続した数値を入れていく

    Args:
        links(list): リンクのリスト. ex) [{'link', 'http://...', 'title': 'hogehoge...'}, {'link': '...', 'title': '...'}, ... ]
    Returns:
        result(list):  [{'link', 'http://...', 'title': 'hogehoge...', num: 1}, {'link': '...', 'title': '...', num: 2}, ... ]
    """
    # result(list)の生成
    result = list()

    num = 1
    for d in links:
        d["num"] = num
        num += 1
        result.append(d)

    return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================

"""engine_yandex
    * Yandex(yandex.com)用の検索用Classを持つモジュール.
"""

from .common import Color
from .engine_common import CommonEngine


class Yandex(CommonEngine):
    """DuckDuckGo

    DuckDuckGo用の検索エンジン用Class.
    """

    def __init__(self):
        None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================

"""common
    * commandでのhelp messageを英語・日本語対応させるためのテキストデータを持つモジュール.
"""

import os

lang = os.getenv('LANG')

if lang == 'ja_JP.UTF-8':
    description = "各種検索エンジンから指定したクエリの結果(url)およびSuggestを取得する"

    # common_args_map
    help_message_query = "検索文字列(クエリ)"
    help_message_op_file = "検索文字列(クエリ)が書かれているファイル"
    help_message_op_template_file = "検索文字列(クエリ)が書かれているテンプレートファイル(jinja2)"
    help_message_op_template_variable = "テンプレートファイル(jinja2)で使用する変数セット(json)"
    help_message_op_search_type = "使用する検索エンジンを指定"
    help_message_op_lang = "言語を指定"
    help_message_op_country = "国を指定"
    help_message_op_proxy_server = "プロキシサーバーを指定(例:socks5://hogehoge:8080, https://fugafuga:18080)"
    help_message_op_json = "json形式で出力する"
    help_message_op_insecure = "sslエラーを無視する"
    help_message_op_selenium = "Selenium(headless browser)を使用する(排他: Splashより優先)"
    help_message_op_splash = "Splash(headless browser)を使用する(排他: Seleniumの方が優先)"
    help_message_op_browser_endpoint = "Selenium/Splash等のヘッドレスブラウザのエンドポイントを指定(例: localhost:8050)"
    help_message_op_browser = "Seleniumで使用するBrowserを指定"
    help_message_op_color = "color出力の切り替え"
    help_message_op_cookies_dir = "使用するcookieファイルの格納先ディレクトリのPATH(各検索エンジンごとでcookieファイルを個別保存)"
    help_message_op_delete_cookies = "検索クエリ実行ごとにCookieを削除する"

    # other_map
    help_message_op_title = "検索結果のタイトルをセットで出力する"
    help_message_op_null_char = "null characterを区切り文字として使用する"
    help_message_op_num = "検索結果の取得数を指定する"
    help_message_op_debug = "debugモードを有効にする"
    help_message_op_disable_headless = "Seleniumでheadlessモードを無効化する(手動でのReCaptcha対応時に必要)"
    help_message_op_start = "期間指定(開始)"
    help_message_op_end = "期間指定(終了)"
    help_message_op_image_pagelink = "画像ファイルがあるhtmlのURLも出力する"

    # suggest_map
    help_message_op_suggest_jap = "日本語の文字を検索キーワードに追加してサジェストを取得"
    help_message_op_suggest_alph = "アルファベット文字を検索キーワードに追加してサジェストを取得"
    help_message_op_suggest_num = "数字を検索キーワードに追加してサジェストを取得"


else:
    description = "Obtain results (url) and Suggest for a specified query from various search engines"

    # common_args_map
    help_message_query = "search string(query)"
    help_message_op_file = "File containing search strings(queries)"
    help_message_op_template_file = "Template file (jinja2) containing search strings (queries)"
    help_message_op_template_variable = "Variable set (json) used in template file (jinja2)"
    help_message_op_search_type = "Specify which search engine to use"
    help_message_op_lang = "Specify language"
    help_message_op_country = "Specify country"
    help_message_op_proxy_server = "Specify proxy server(example: socks5://hogehoge:8080, https://fugafuga:18080)"
    help_message_op_json = "Output in json format"
    help_message_op_insecure = "ignore ssl errors"
    help_message_op_selenium = "Use Selenium (headless browser). (exclusive: takes precedence over Splash)"
    help_message_op_splash = "Use Splash (headless browser) (exclusive: Selenium is preferred)"
    help_message_op_browser_endpoint = "Specify the endpoint for headless browsers such as Selenium/Splash (example: localhost:8050)"
    help_message_op_browser = "Specify Browser to use with Selenium"
    help_message_op_color = "Switching color output"
    help_message_op_cookies_dir = "PATH of the directory where the cookie files to be used are stored (cookie files are stored separately for each search engine)"
    help_message_op_delete_cookies = "Delete cookies on every search query execution"

    # other_map
    help_message_op_title = "Output a set of search result titles"
    help_message_op_null_char = "Use null character as delimiter"
    help_message_op_num = "Specify the number of search results to retrieve"
    help_message_op_debug = "Enable debug mode"
    help_message_op_disable_headless = "Disable headless mode in Selenium (required for manual ReCaptcha support)"
    help_message_op_start = "Search period (start)"
    help_message_op_end = "Search period (end)"
    help_message_op_image_pagelink = "Also output the html URL where the image files are located."

    # suggest_map
    help_message_op_suggest_jap = "Add Japanese characters to search keywords to get suggestions"
    help_message_op_suggest_alph = "Add alphabetic characters to search keywords to get suggestions"
    help_message_op_suggest_num = "Add numbers to search keywords to get suggestions"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""engine_yahoo
    * Yahoo(yahoo.co.jp)用の検索用Classを持つモジュール.
"""


import json
import re
import sys

from urllib import parse
from bs4 import BeautifulSoup

from .common import Color
from .engine_common import CommonEngine


class Yahoo(CommonEngine):
    """Yahoo

    Yahoo(yahoo.co.jp)用の検索エンジン用Class.
    """

    def __init__(self):
        # CommonEngineの処理を呼出し
        super().__init__()

        self.NAME = 'Yahoo'
        self.COLOR = Color.YELLOW
        self.COLOR_NAME = self.COLOR + self.NAME + Color.END

        # リクエスト先のURLを指定
        self.ENGINE_TOP_URL = 'https://www.yahoo.co.jp/'
        self.SEARCH_URL = 'https://search.yahoo.co.jp/search'
        self.IMAGE_PRE_URL = 'https://search.yahoo.co.jp/image/search'
        self.IMAGE_URL = 'https://search.yahoo.co.jp/image/api/search'
        self.SUGGEST_URL = 'https://ff.search.yahoo.com/gossip'

    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: 検索用url
        """
        search_url = ''

        # 検索タイプがtextの場合
        if type == 'text':
            # 検索urlを指定
            search_url = self.SEARCH_URL

            # 検索パラメータの設定
            url_param = {
                'p': keyword,         # 検索キーワード
                'num': '100',    # 指定不可(削除)
                'day_from': '',  # 開始日時(yyyy/mm/dd)
                'day_to': '',    # 終了日時(yyyy/mm/dd)
                'b': '',         # 開始位置
                'nfpr': '1',     # もしかして検索(Escape hatch)の無効化
                'qrw': '0'       # もしかして検索(Escape hatch)の無効化
            }

            # lang/localeが設定されている場合
            if self.LANG != '' and self.LOCALE != '':
                url_param['hl'] = self.LANG
                url_param['gl'] = self.LOCALE

            # rangeが設定されている場合
            try:
                start = self.RANGE_START
                end = self.RANGE_END

                # ex.) day_from=2019/09/01&day_to=2019/09/30
                # パラメータが2つ存在している
                day_from = start.strftime("%Y/%m/%d")
                day_to = end.strftime("%Y/%m/%d")

                # GETパラメータに日時データを追加
                url_param['day_from'] = day_from
                url_param['day_to'] = day_to

            except AttributeError:
                None

        # 検索タイプがimageの場合
        elif type == 'image':
            # 前処理(パラメータ`cr`の取得)を実行
            cr = self.get_image_search_cr(keyword)

            # 検索urlを指定
            search_url = self.IMAGE_URL

            # 検索パラメータの設定
            url_param = {
                'p': keyword,  # 検索キーワード
                'fr': 'top_ga1_sa',
                'ei': 'UTF-8',
                'aq': '-1',
                'n': '20',  # 指定不可(削除)
                'vm': 'i',
                'se': '0',
                'ue': '0',
                'cr': cr,
                # 'day_from': '',  # 開始日時(yyyy/mm/dd)
                # 'day_to': '',    # 終了日時(yyyy/mm/dd)
                'b': '',         # 開始位置
                'nfpr': '1',     # もしかして検索(Escape hatch)の無効化
                'qrw': '0'       # もしかして検索(Escape hatch)の無効化
            }

        page = 0
        while True:
            # parameterにページを開始する番号を指定
            if type == 'text':
                url_param['b'] = str(page * 10)
            elif type == 'image':
                url_param['b'] = str(page * 10)

            # パラメータをセット
            params = parse.urlencode(url_param)

            target_url = search_url + '?' + params

            yield 'GET', target_url, None

            page += 1

    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """
        url_param = {
            'command': keyword,   # 検索キーワード
            'output': 'json',
        }

        params = parse.urlencode(url_param)
        url = self.SUGGEST_URL + '?' + params

        return url

    def get_links(self, url: str, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            url  (str): 解析する検索結果のurl.
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'url': 'https://hogehoge....'}, {...}]`)
        """

        if type == 'text':
            if self.USE_SPLASH or self.USE_SELENIUM:
                self.SOUP_SELECT_JSON = '#__NEXT_DATA__'
                self.SOUP_SELECT_IMAGE = '.rg_meta.notranslate'
                self.SOUP_SELECT_TEXT = ''

                # Yahooの場合、jsonから検索結果を取得する
                soup = BeautifulSoup(html, 'lxml')
                elements = soup.select(self.SOUP_SELECT_JSON)
                element = elements[0].string

                # debug
                if self.IS_DEBUG:
                    print(Color.PURPLE + '[JsonElement]' + Color.END,
                          file=sys.stderr)
                    print(Color.PURPLE + element + Color.END,
                          file=sys.stderr)  # type: ignore

                # jsonからデータを抽出　
                j = json.loads(element)  # type: ignore

                # debug
                if self.IS_DEBUG:
                    print(Color.PURPLE + '[Json]' + Color.END, file=sys.stderr)
                    print(Color.PURPLE + json.dumps(j) + Color.END,
                          file=sys.stderr)

                jd = j['props']['initialProps']['pageProps']['pageData']['algos']

                elinks = [e['url'] for e in jd]
                etitles = [e['title'] for e in jd]
                etexts = [e['description'] for e in jd]

                links = self.create_text_links(url, elinks, etitles, etexts)

            else:
                self.SOUP_SELECT_URL = '.sw-Card__headerSpace > .sw-Card__title > a'
                self.SOUP_SELECT_TITLE = '.sw-Card__headerSpace > .sw-Card__title > a > h3'
                self.SOUP_SELECT_TEXT = '.sw-Card__floatContainer > .sw-Card__summary'

                # CommonEngineの処理を呼び出す
                links = super().get_links(url, html, type)

        elif type == 'image':
            # CommonEngineの処理を呼び出す
            links = super().get_links(url, html, type)

        return links

    # 画像検索ページの検索結果(links(list()))を生成するfunction
    def get_image_links(self, soup: BeautifulSoup):
        """get_image_links
        BeautifulSoupから画像検索ページを解析して結果を返す関数.

        Args:
            soup (BeautifulSoup): 解析するBeautifulSoupオブジェクト.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]`)
        """

        result = []  # image url

        try:
            data = json.loads(soup.text)
        except Exception:
            return result

        for d in data['algos']:
            etitle = d['title']
            elink = d['refererUrl']
            eimage = d['original']['url']

            el = {
                'title': etitle,
                'pagelink': elink,
                'link': eimage,
            }

            result.append(el)

        return result

    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """
        if self.USE_SELENIUM and self.SELENIUM_BROWSER == 'firefox':
            soup = BeautifulSoup(html, features="lxml")
            html = soup.find("pre").text
        data = json.loads(html)
        suggests[char if char == '' else char[-1]] = [e['key']  # type: ignore
                                                      for e in data['gossip']['results']]

        return suggests

    def get_image_search_cr(self, keyword: str):
        """get_image_search_cr

        Yahooの画像検索時に必要になるcrumb(cr)パラメータを取得するための前処理リクエストを行う関数

        Args:
            keyword (str): 検索キーワード

        Returns:
            str: crumbパラメータの値
        """

        result = ''

        # urlパラメータを設定
        url_param = {
            'p': keyword,
            'fr': 'top_ga1_sa',
            'ei': 'UTF-8',
            'aq': '-1',
        }
        params = parse.urlencode(url_param)

        # 前処理リクエストを投げる
        pre_result = self.get_result(self.IMAGE_PRE_URL + '?' + params)

        # 前処理リクエストから、crumbパラメータの値を取得する(正規表現)
        pattern = r'{ *"crumb": *"[^"]+" *}'
        data = re.findall(pattern, pre_result)

        if len(data) > 0:
            d = data[0]
            jd = json.loads(d)

            result = jd['crumb']

        return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""engine_bing
    * Bing用の検索用Classを持つモジュール.
"""

import requests
import datetime
import json
import asyncio
import re

from urllib import parse
from bs4 import BeautifulSoup

from .common import Color
from .engine_common import CommonEngine


class Bing(CommonEngine):
    """Bing

    Bing用の検索エンジン用Class.
    """

    def __init__(self):
        # CommonEngineの処理を呼出し
        super().__init__()

        self.NAME = 'Bing'
        self.COLOR = Color.CYAN
        self.COLOR_NAME = self.COLOR + self.NAME + Color.END

        # リクエスト先のURLを指定
        self.ENGINE_TOP_URL = 'https://www.bing.com/'
        self.SEARCH_URL = 'https://www.bing.com/search'
        self.IMAGE_URL = 'https://www.bing.com/images/async'
        self.SUGGEST_URL = 'https://www.bing.com/AS/Suggestions'

    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: 検索用url
        """

        search_url = ''

        # NOTE:
        #   2023/07/27にて、queryが以下のように切り替わったため修正
        #     - `https://www.bing.com/search?q=site%3aorebibou.com&search=%e9%80%81%e4%bf%a1&rdr=1&rdrig=D4B6730A85514F25BAE1E9BDC04F1C28&cc=us&setlang=en`
        #       ```json
        #       {
        #         'q': ['site:orebibou.com'],
        #         'search': ['送信'],
        #         'rdr': ['1'],
        #         'rdrig': ['D4B6730A85514F25BAE1E9BDC04F1C28'],
        #         'cc': ['us'],
        #         'setlang': ['en']
        #       }
        #       ```
        #     - `https://www.bing.com/search?q=site%3aorebibou.com&search=%E9%80%81%E4%BF%A1&rdr=1&rdrig=D4B6730A85514F25BAE1E9BDC04F1C28&cc=us&setlang=en&FPIG=B035C5DE50AE4A328CB93C767B02D08B&first=11&FORM=PERE&count=100`

        # 検索タイプがtextの場合
        if type == 'text':
            # 検索urlを指定
            search_url = self.SEARCH_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,    # 検索キーワード
                'count': '100',  # 1ページごとの表示件数
                'search': '送信',
                'rdr': '1',
                'from': 'PERE',
                'cc': 'us',
                'setlang': 'en',
                'filters': '',   # 期間含めフィルターとして指定するパラメータ
                'first': ''      # 開始位置
            }

            # lang/localeが設定されている場合
            if self.LANG != '':
                url_param['setlang'] = self.LANG.lower()
            if self.LOCALE != '':
                url_param['cc'] = self.LOCALE.lower()

            # rangeが設定されている場合
            try:
                start = self.RANGE_START
                end = self.RANGE_END

                unix_day = datetime.strptime('1970-01-01', "%Y-%m-%d")
                cd_min = (start - unix_day).days
                cd_max = (end - unix_day).days

                # GETパラメータに日時データを追加
                url_param['filters'] = 'ex1:"ez5_{0}_{1}"'.format(
                    cd_min, cd_max)

            except AttributeError:
                None

        # 検索タイプがimageの場合
        elif type == 'image':
            # 検索urlを指定
            search_url = self.IMAGE_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,    # 検索キーワード
                'count': '100',  # 1回ごとの件数
                'first': '',     # 検索位置
                'tsc': 'ImageBasicHover',
                'layout': 'RowBased',
            }

            # rangeが指定されている場合
            # TODO: 日時パラメータを追加(ex: `qft=+filterui%3aage-lt43200`)

        page = 0
        while True:
            # parameterにページを開始する番号を指定
            url_param['first'] = str(page * 100)
            params = parse.urlencode(url_param)

            target_url = search_url + '?' + params

            yield 'GET', target_url, None

            page += 1

    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """

        url_param = {
            'qry': keyword,  # 検索キーワード
            'cvid': 'F5F47E4155E44D86A86690B49023B0EF'
        }

        params = parse.urlencode(url_param)
        url = self.SUGGEST_URL + '?' + params

        return url

    def get_links(self, url: str, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'url': 'https://hogehoge....'}, {...}]`)
        """

        if type == 'text':
            self.SOUP_SELECT_URL = 'h2 > a'
            self.SOUP_SELECT_TITLE = 'h2 > a'
            self.SOUP_SELECT_TEXT = 'li > div > p'

        elif type == 'image':
            self.SOUP_SELECT_URL = '.imgpt > .iusc'

        # CommonEngineの処理を呼び出す
        links = super().get_links(url, html, type)

        return links

    # 画像検索ページの検索結果(links(list()))を生成するfunction
    def get_image_links(self, soup: BeautifulSoup):
        """get_image_links
        BeautifulSoupから画像検索ページを解析して結果を返す関数.

        Args:
            soup (BeautifulSoup): 解析するBeautifulSoupオブジェクト.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]`)
        """

        elements = soup.select(self.SOUP_SELECT_URL)
        edata = [e['m'] for e in elements]

        result = []  # image url
        for e in edata:
            # json化
            je = json.loads(e)

            etitle = je['t']
            elink = je['purl']
            eimage = je['murl']

            el = {
                'title': etitle,
                'pagelink': elink,
                'link': eimage,
            }

            result.append(el)

        return result

    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """
        soup = BeautifulSoup(html, 'lxml')
        elements = soup.select('ul > li')
        suggests[char if char == '' else char[-1]] = [e['query']
                                                      for e in elements]
        return suggests

    def processings_elist(self, elinks, etitles, etexts: list):
        """processings_elist

        self.get_links 内で、取得直後のelinks, etitlesに加工を加えるための関数.
        requestsを用いて、リダイレクトリンクから遷移先urlを取得していく.

        Args:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列

        Returns:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列
        """

        # 通常のスクレイピングとは別にセッションを作成
        session = requests.session()

        # pool sizeを調整
        adapter = requests.adapters.HTTPAdapter(
            pool_connections=100, pool_maxsize=100)
        session.mount('https://', adapter)

        # proxyを設定
        if self.PROXY != '':
            proxies = {
                'http': self.PROXY,
                'https': self.PROXY
            }
            session.proxies = proxies

        # user-agentを設定
        if self.USER_AGENT != '':
            session.headers.update(
                {
                    'User-Agent': self.USER_AGENT,
                    'Accept-Language': 'ja,en-US;q=0.7,en;q=0.3'
                }
            )

        # asyncio loopを作成
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        # リダイレクト先のurlに置き換え
        elinks = loop.run_until_complete(
            resolv_links(loop, session, elinks))
        loop.close()

        return elinks, etitles, etexts


async def resolv_links(loop: asyncio.AbstractEventLoop, session: requests.Session, links: list):
    """resolv_links

    リダイレクト先のurlをパラレルで取得する(Baiduで使用)

    Args:
        loop (asyncio.AbstractEventLoop): loop
        session (requests.Session): 使用するSession
        links (list): リダイレクト先を取得するurlのリスト

    Returns:
        data (list): リダイレクト先を取得したurlのリスト
    """

    async def req(session: requests.Session, url: str):
        task = await loop.run_in_executor(None, resolv_url, session, url)
        return task

    tasks = []
    for link in links:
        # urlをパース
        url = parse.urlparse(link)

        # bingの遷移ページの場合はリダイレクトして処理
        if url.netloc == 'www.bing.com' and url.path == '/ck/a':
            task = req(session, link)
            tasks.append(task)

    data = await asyncio.gather(*tasks)

    return data


def resolv_url(session: requests.Session, url: str):
    """resolv_url
    リダイレクト先のurlを取得する(Baiduで使用)
    Args:
        session (request.Session): リダイレクト先を取得する際に使用するSession
        url (str): リダイレクト先を取得するurl
    Returns:
        url (str): リダイレクト先のurl
    """

    while True:
        try:
            # リダイレクト先のbodyを取得する
            res = session.get(url).text

        except requests.RequestException:
            continue
        except ConnectionError:
            continue
        else:
            # resから１行ずつチェック
            for line in res.splitlines():
                if re.match('^ +var u', line):
                    text = re.findall('"([^"]*)"', line)
                    url = text[0]
                    break
            break

    return url



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""engine_duckduckgo
    * DuckDuckGo用の検索用Classを持つモジュール.
"""


import json
import re
import sys

from time import sleep
from urllib import parse
from bs4 import BeautifulSoup

from .common import Color
from .engine_common import CommonEngine


class DuckDuckGo(CommonEngine):
    """DuckDuckGo

    DuckDuckGo用の検索エンジン用Class.
    """

    def __init__(self):
        # CommonEngineの処理を呼出し
        super().__init__()

        self.NAME = 'DuckDuckGo'
        self.COLOR = Color.BLUE
        self.COLOR_NAME = self.COLOR + self.NAME + Color.END

        # リクエスト先のURLを指定
        self.PRE_URL = 'https://duckduckgo.com/'
        self.ENGINE_TOP_URL = 'https://duckduckgo.com/'
        self.SEARCH_URL = 'https://links.duckduckgo.com/d.js'
        self.IMAGE_URL = 'https://duckduckgo.com/i.js'
        self.SUGGEST_URL = 'https://duckduckgo.com/ac/'

    def request_selenium(self, url: str, method='GET', data=None):
        if self.SUGGEST_URL in url:
            # 最初にTOPページを表示
            self.driver.get(self.ENGINE_TOP_URL)

            self.driver.implicitly_wait(3)

            # javascriptからリクエストを投げてjsonを取得
            exec_java_script = 'return fetch("{}").then(response=>response.json())'.format(
                url)
            result = self.driver.execute_script(exec_java_script)

            result = json.dumps(result)

        else:
            result = super().request_selenium(url, method, data)

        return result

    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: 検索用url
        """

        # 前処理リクエスト用パラメータの設定
        pre_param = {
            'q': keyword,  # 検索キーワード
            't': 'h_'
        }

        try:
            # 前処理リクエスのセッションを生成する
            pre_params = parse.urlencode(pre_param)
            pre_url = self.PRE_URL + '?' + pre_params

            # 前処理リクエスト1を実行
            self.get_result('https://duckduckgo.com/?t=h_')

            # 待機時間を入れる
            sleep(1)

            # 前処理リクエスト2を実行
            pre_html = self.get_result(pre_url)
            sleep(1)

            r = re.findall(
                r"(?<=vqd\=)[0-9-]+", pre_html
            )

            # get vqd
            vqd = r[0]

        except Exception:
            return

        if type == 'text':
            # 検索urlを指定
            search_url = self.SEARCH_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,  # 検索キーワード
                's': 0,  # 取得開始件数
                'vqd': vqd
            }

            # lang/localeが設定されている場合
            if self.LANG != '' and self.LOCALE != '':
                url_param['l'] = self.LANG + '_' + self.LOCALE

            # rangeが設定されている場合(DuckDuckGoにはレンジ指定がないらしいので、追加されたら記述する)

        elif type == 'image':
            # 検索urlを指定
            search_url = self.IMAGE_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,  # 検索キーワード
                'o': 'json',  # output format
                'p': 1,
                's': 0,  # 取得開始件数
                'u': 'bing',  # TODO: 利用する検索エンジン(おそらく).後でオプションで指定できるようにする.
                'f': ',,,,,',
                'vqd': vqd
            }

            # lang/localeが設定されている場合
            if self.LANG != '' and self.LOCALE != '':
                url_param['l'] = self.LANG + '-' + self.LANG

        # set next_url
        params = parse.urlencode(url_param)
        self.next_url = search_url + '?' + params

        # while loop
        page = 0
        while True:
            if self.next_url == "":
                break

            # get next_url
            target_url = self.next_url

            yield 'GET', target_url, None

            page += 1

    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """
        url_param = {
            'q': keyword,  # 検索キーワード
            'kl': 'wt-wt'
        }

        params = parse.urlencode(url_param)
        url = self.SUGGEST_URL + '?' + params

        return url

    def get_links(self, source_url: str, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            url  (str): 解析する検索結果のurl.
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'url': 'https://hogehoge....'}, {...}]`)
        """
        links = list()

        # next_url用のurl
        url = ""
        vqd = ""

        if type == 'text':
            # 加工してdictとして扱えるようにする
            r = re.findall(
                r"DDG\.pageLayout\.load\(\'d\',(.+)\]\)\;", html
            )

            try:
                r_dict = json.loads(r[0] + "]")
            except Exception:
                return links

            for r_data in r_dict:
                if "u" in r_data and "s" in r_data:
                    d = {
                        "link": r_data["u"],
                        "title": BeautifulSoup(
                            r_data["t"], "lxml").text,
                        "text": BeautifulSoup(
                            r_data["a"], "lxml").text,
                        "source_url": source_url,
                    }
                    links.append(d)

                elif "n" in r_data:
                    base_uri = '{uri.scheme}://{uri.netloc}'.format(
                        uri=parse.urlparse(self.SEARCH_URL)
                    )
                    url = base_uri + r_data["n"]

        elif type == 'image':
            # seleniumを使用している場合、htmlを上書き
            if self.USE_SELENIUM or self.USE_SPLASH:
                soup = BeautifulSoup(html, "lxml")
                selected_one = soup.select_one('html > body > pre')
                html = selected_one.text

            # jsonとして読み込む
            try:
                data = json.loads(html)
            except Exception as e:
                print(e, file=sys.stderr)
                return links

            if 'results' in data:
                results = data['results']

                for r in results:
                    d = {
                        'link': r['image'],
                        'title': r['title'],
                        'pagelink': r['url']
                    }
                    links.append(d)

            if 'vqd' in data:
                vqd = list(data['vqd'].values())[0]

            # next_url用のurlを取得する
            if 'next' in data:
                next_path = data['next']
                next_path = next_path + '&vqd=' + vqd
                base_url = '{uri.scheme}://{uri.netloc}/'.format(
                    uri=parse.urlparse(self.IMAGE_URL)
                )
                url = base_url + next_path

        if url != "":
            self.next_url = url

        return links

    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """

        data = json.loads(html)
        suggests[char if char == '' else char[-1]] = [e['phrase']
                                                      for e in data]

        return suggests



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""engine_google
    * Google用の検索用Classを持つモジュール.
"""


import json
import os

from time import sleep
from json.decoder import JSONDecodeError
from urllib import parse
from lxml import etree
from bs4 import BeautifulSoup

from .common import Color
from .recaptcha import TwoCaptcha
from .engine_common import CommonEngine


# Google画像検索で使用するパラメータID
RPC_ID = "HoAMBc"


class Google(CommonEngine):
    """Google

    Google用の検索エンジン用Class.
    """

    def __init__(self):
        # CommonEngineの処理を呼出し
        super().__init__()

        self.NAME = 'Google'
        self.COLOR = Color.PURPLE
        self.COLOR_NAME = self.COLOR + self.NAME + Color.END

        # リクエスト先のURLを指定
        self.ENGINE_TOP_URL = 'https://www.google.com/'
        self.SEARCH_URL = 'https://www.google.com/search'
        self.IMAGE_URL = 'https://www.google.com/_/VisualFrontendUi/data/batchexecute'
        self.SUGGEST_URL = 'http://www.google.com/complete/search'

        # 次の検索ページのURL(`self.get_nextpage_url`の処理で取得する)
        self.SEARCH_NEXT_URL = None

        # ReCaptcha画面かどうかの識別用
        self.SOUP_RECAPTCHA_TAG = '#captcha-form > #recaptcha'

    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: 検索用url
        """

        search_url = ''

        if type == 'text':
            # 検索用urlを指定
            search_url = self.SEARCH_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,   # 検索キーワード
                'oq': keyword,  # 検索キーワード
                'num': '100',   # 1ページごとの表示件数.
                'filter': '0',  # 類似ページのフィルタリング(0...無効, 1...有効)
                'start': '',    # 開始位置
                'tbs': '',      # 期間
                'nfpr': '1'     # もしかして検索(Escape hatch)を無効化
            }

            # lang/localeが設定されている場合
            if self.LANG != '' and self.LOCALE != '':
                url_param['hl'] = self.LANG
                url_param['gl'] = self.LOCALE

            # rangeが設定されている場合
            try:
                start = self.RANGE_START
                end = self.RANGE_END

                cd_min = start.strftime("%m/%d/%Y")
                cd_max = end.strftime("%m/%d/%Y")

                # GETパラメータに日時データを追加
                url_param['tbs'] = "cdr:1,cd_min:{0},cd_max:{1}".format(
                    cd_min, cd_max)

            except AttributeError:
                None

            page = 0
            while True:
                if page == 0:
                    # parameterにページを開始する番号を指定
                    url_param['start'] = str(page * 100)
                    params = parse.urlencode(url_param)

                    target_url = search_url + '?' + params

                else:
                    target_url = self.SEARCH_NEXT_URL
                    if self.SEARCH_NEXT_URL is None:
                        break

                yield 'GET', target_url, None
                page += 1

        elif type == 'image':
            # 検索用urlを指定
            search_url = self.IMAGE_URL

            # Refererの設定
            if not self.USE_SELENIUM:
                self.session.headers.update(
                    {"Referer": "https://www.google.com/"}
                )

            # 検索パラメータの設定
            url_param = {
                'rpcids': 'HoAMBc',
                'hl': 'id',
                'authuser': '0',
                'soc-app': '162',
                'soc-platform': '1',
                'soc-device': '1',
                'rt': 'c'
            }

            # 画像のカーソル位置指定パラメータを作成
            self.image_next_cursor = None
            self.image_cursor = []

            page = 0
            while True:
                # post dataを生成
                data = {
                    "f.req": build_rpc_request(keyword, (self.image_cursor, self.image_next_cursor), page),
                    "at": "ABrGKkQnVYg89U_cdKuhNZ5hM4vx:1616119655028",
                    # "": "",
                }

                params = parse.urlencode(url_param)
                target_url = search_url + '?' + params

                yield 'POST', target_url, data

    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """

        url_param = {
            'q': keyword,  # 検索キーワード
            'output': 'toolbar',
            'ie': 'utf-8',
            'oe': 'utf-8',
        }

        params = parse.urlencode(url_param)
        url = self.SUGGEST_URL + '?' + params

        return url

    def get_links(self, url: str, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            url  (str): 解析する検索結果のurl.
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果。変数名はlinks。(`[{'title': 'title...', 'url': 'https://hogehoge....'}, {...}]`)
        """

        # テキスト検索の場合
        if type == 'text':
            # request or seleniumの定義
            self.SOUP_SELECT_URL = '#main > div > div > .kCrYT > a'
            self.SOUP_SELECT_TITLE = '#main > div > div > .kCrYT > a > h3 > div'
            self.SOUP_SELECT_TEXT = '#main > div > div > .kCrYT > div > div > div > div > div'
            self.SOUP_SELECT_NEXT_URL = ''

            # Selenium経由、かつFirefoxを使っている場合
            if self.USE_SELENIUM:
                self.SOUP_SELECT_URL = '.yuRUbf > a'
                self.SOUP_SELECT_TITLE = '.yuRUbf > a > .LC20lb'
                self.SOUP_SELECT_TEXT = '.lEBKkf'
                self.SOUP_SELECT_NEXT_URL = '.d6cvqb > a'

            # Splash経由で通信している場合
            elif self.USE_SPLASH:
                self.SOUP_SELECT_URL = '.yuRUbf > a'
                self.SOUP_SELECT_TITLE = '.yuRUbf > a > .LC20lb'
                self.SOUP_SELECT_TEXT = '.lEBKkf'
                self.SOUP_SELECT_NEXT_URL = '.d6cvqb > a'

            # TODO: SEARCH_NEXT_URLを書き換える
            self.get_nextpage_url(html)

            # CommonEngineの処理を呼び出す
            links = super().get_links(url, html, type)

        # イメージ検索の場合
        elif type == 'image':
            links = self.get_image_links(html)

        return links

    def get_image_links(self, html: str):
        """get_image_links

        BeautifulSoupから画像検索ページを解析して結果を返す関数.
        Seleniumを利用し、自動的にページ末尾まで移動して続きを取得する.
        クリック等が発生するため、抽出にかなり時間がかかる.

        Args:
            html (str): 解析する検索結果のhtml.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]`)

        参考:
            - https://github.com/Wikidepia/py-googleimages/blob/b781b79e9bf40d29cf6fcbdcf625303abf3718bd/googleimages/client.py
        """

        links = list()

        # 改行区切りでloop
        for line in html.split("\n"):
            if RPC_ID not in line:
                continue

            # Make it json readable
            line_cl = line.replace("\\n", "")  # Remove \n

        lineson = json.loads(line_cl)

        data = pjson_loads(lineson[0][2])

        # 画像のカーソル位置を更新
        self.image_next_cursor = data[-2]
        self.image_img_cursor = data[31][0][12][11][5]

        for img in data[31][0][12][2]:
            # imgの値をチェック
            if img[1] is None:
                continue

            link = img[1][3][0]  # 画像ファイルのurl
            title = img[1][9]['2003'][3]  # 画像ファイルのあるページのtitle
            pagelink = img[1][9]['2003'][2]  # 画像ファイルのあるページのurl
            links.append(
                {
                    "link": link,
                    "title": title,
                    "pagelink": pagelink,
                }
            )

        return links

    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """

        sug_root = etree.XML(html)
        sug_data = sug_root.xpath("//suggestion")
        data = [s.get("data") for s in sug_data]

        suggests[char if char == '' else char[-1]] = data  # type: ignore

        return suggests

    def get_nextpage_url(self, html: str):
        # BeautifulSoupでの解析を実施
        soup = BeautifulSoup(html, 'lxml')

        # BeautifulSoupでnext urlの要素を確認する
        elements = soup.select(self.SOUP_SELECT_NEXT_URL)

        # next urlを取得する
        elinks = [e['href'] for e in elements]

        if len(elinks) == 0:
            self.SEARCH_NEXT_URL = None

        elif len(elinks) == 1:
            next_url = parse.urljoin(
                self.ENGINE_TOP_URL, elinks[0])  # type: ignore
            self.SEARCH_NEXT_URL = next_url

        elif len(elinks) > 1:
            next_url = parse.urljoin(
                self.ENGINE_TOP_URL, elinks[1])  # type: ignore
            self.SEARCH_NEXT_URL = next_url

    def processings_elist(self, elinks, etitles, etexts: list):
        """processings_elist

        self.get_links 内で、取得直後のelinks, etitlesに加工を加えるための関数.

        Args:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列

        Returns:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列
        """

        # seleniumでfirefoxを使っていない、かつsplashを使っていない場合
        new_elinks = []
        for elink in elinks:
            parsed = parse.urlparse(elink)
            parsed_query = parse.parse_qs(parsed.query)

            if 'url' in parsed_query and elink[0] == '/':
                parsed_q = parsed_query['url']
                if len(parsed_q) > 0:
                    new_elink = parsed_q[0]
                    new_elinks.append(new_elink)
            else:
                new_elinks.append(elink)
        elinks = list(dict.fromkeys(new_elinks))

        return elinks, etitles, etexts

    def bypass_recaptcha_selenium(self, url: str, html: str):
        """bypass_recaptcha_selenium

        SeleniumでReCaptchaを突破する関数.
        2Captchaでの自動突破の場合、CookieとProxyが必要となる.

        Args:
            url (str): ReCaptcha画面が表示されてしまったリクエストのurl
            html (str): ReCaptcha画面のhtml

        Returns:
            str: ReCaptchaを突破後のurlのhtml
        """

        # resultを定義しておく
        result = None

        # 環境変数を取得
        TC_API_KEY = os.getenv('API_KEY_2CAPTCHA')

        # Seleniumの場合、手動でBypassが行えるようにする
        if self.IS_DISABLE_HEADLESS:
            while True:
                # 現在のSeleniumのurlを取得する
                current_url = self.driver.current_url
                current_url_parse = parse.urlparse(current_url)

                # current_urlのpathが `/sorry/index` か識別する
                current_url_path = current_url_parse.path
                if current_url_path != '/sorry/index':
                    break

                # 待機
                sleep(1)

            sleep(5)

            # 現在のページ(ReCaptchaから移動したページ)のhtmlを取得する
            result = self.driver.page_source

        # self.IS_DISABLE_HEADLESS がFalseで、かつ`API_KEY_2CAPTCHA`が定義されている場合
        elif TC_API_KEY is not None:
            # solverを作成
            solver = TwoCaptcha(TC_API_KEY)

            # flag set
            solver.set_debug(self.IS_DEBUG)
            solver.set_command(self.IS_COMMAND)
            solver.set_user_agent(self.USER_AGENT)
            solver.set_messages(self.MESSAGE)

            # solverからのレスポンスを取得する
            code = solver.google_recaptcha(
                html=html,
                url=url,
                cookies=self.driver.get_cookies(),
                proxy=self.PROXY,

            )

            # ReCaptchaの解除に失敗した場合
            if code is None:
                return result

            # 解除コードを所定のtextareaに入力
            self.driver.execute_script("""
              document.getElementById(
                  "g-recaptcha-response").innerHTML = arguments[0]
            """, code)

            # ボタンクリック
            self.driver.execute_script(
                'var element=document.getElementById("g-recaptcha-response"); element.style.display="none";')

            self.driver.execute_script('submitCallback()')

            sleep(10)

            # 結果を取得する
            result = self.driver.page_source

        return result


def build_rpc_request(keyword: str, cursor: list, page: int):
    """build_rpc_request

    画像検索で使用するrpcデータの生成用関数.

    Original:
        https://github.com/Wikidepia/py-googleimages/blob/b781b79e9bf40d29cf6fcbdcf625303abf3718bd/googleimages/utils.py

    Args:
        keyword (str): [description]
        cursor (list): [description]
        page (int): [description]

    Returns:
        [type]: [description]
    """

    RPC_ID = "HoAMBc"

    return json.dumps(
        [
            [
                [
                    RPC_ID,
                    json.dumps(
                        [
                            None,
                            None,
                            [
                                1,
                                None,
                                450,
                                1,
                                1280,
                                cursor[0],
                                [],
                                [],
                                None,
                                None,
                                None,
                                0,
                                310,
                                [],
                            ],
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            [
                                keyword,
                                None,
                                None,
                                "strict",
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                "lnms",
                            ],
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            [
                                cursor[1],
                                "CAM=",
                                "CgtHUklEX1NUQVRFMBAaIAA=",
                            ],
                        ],
                        separators=(",", ":"),
                    ),
                    None,
                    "generic",
                ],
            ]
        ],
        separators=(",", ":"),
    )


def pjson_loads(text):
    """pjson_loads

    画像検索で使用するデータの生成用関数.

    Original:
        https://github.com/Wikidepia/py-googleimages/blob/b781b79e9bf40d29cf6fcbdcf625303abf3718bd/googleimages/utils.py

    Args:
        text ([type]): [description]

    Returns:
        [type]: [description]
    """
    while True:
        try:
            data = json.loads(text, strict=False)
        except JSONDecodeError as exc:
            if exc.msg == "Invalid \\escape":
                text = text[: exc.pos] + "\\" + text[exc.pos:]
            else:
                raise
        else:
            return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""engine_common
    * SearchEngine Classから呼び出す、各検索エンジンで共通の処理を保持させる継承用Classである `CommonEngine` を持つモジュール.
"""


import requests
import os
import pickle

# selenium driver auto install packages
import chromedriver_autoinstaller
import geckodriver_autoinstaller

# seleniumrequests
from seleniumrequests import Chrome, Firefox

# selenium
from selenium import webdriver
from selenium.webdriver.chrome.options import Options as ChromeOptions
from selenium.webdriver.firefox.options import Options as FirefoxOptions
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from urllib import parse
from fake_useragent import UserAgent
from bs4 import BeautifulSoup
from datetime import datetime

from .common import Color, Message


# 各検索エンジン用class共通の処理を記述した継承用class
class CommonEngine:
    """CommonEngine

    検索エンジンごとの処理を記述するClassのための、継承用Class.
    """

    # Class作成時の処理
    def __init__(self):
        # headless browserの利用有無フラグ(デフォルト: False)
        self.USE_SELENIUM = False
        self.USE_SPLASH = False

        # 初期値の作成
        self.LOCK = None
        self.COOKIE_FILE = ''
        self.COOKIE_FILE_DELETE = False
        self.SPLASH_URI = ''
        self.PROXY = ''
        self.USER_AGENT = ''
        self.LANG = ''
        self.LOCALE = ''
        self.IS_DEBUG = False
        self.IS_COMMAND = False
        self.IS_DISABLE_HEADLESS = False
        self.MESSAGE: Message
        self.IGNORE_SSL_VERIFY = False

        # ReCaptcha画面かどうかの識別用(初期値(ブランク))
        self.RECAPTCHA_SITEKEY = ''
        self.SOUP_RECAPTCHA_TAG = ''
        self.SOUP_RECAPTCHA_SITEKEY = ''

    # 検索エンジンにわたす言語・国の設定を受け付ける
    def set_lang(self, lang: str, locale: str):
        """set_lang

        検索エンジンで指定する言語・国の設定を行う関数

        Args:
            lang (str): 検索エンジンのパラメータで指定する言語を指定する([ja,en])
            locale (str): 検索エンジンのパラメータで指定する国を指定する([JP,US])
        """

        self.LANG = lang
        self.LOCALE = locale

    # 検索時の日時範囲を指定
    def set_range(self, start: datetime, end: datetime):
        """set_range

        検索エンジンで指定する日付範囲を指定する

        Args:
            start (datetime): 検索対象ページの対象範囲開始日時(datetime)
            end (datetime): 検索対象ページの対象範囲終了日時(datetime)
        """
        self.RANGE_START = start
        self.RANGE_END = end

    # user_agentの設定値を受け付ける(引数がない場合はランダム。Seleniumの際は自動的に使用したbrowserのagentを指定)
    def set_user_agent(self, user_agent: str = None, browser: str = None):  # type: ignore
        """set_user_agent

        user_agentの値を受け付ける.
        user_agentの指定がない場合、 Chromeを使用したものとする.
        また、もし`browser`が指定されている場合はそのブラウザのUser Agentを指定する.

        注) seleniumを利用する場合、事前に有効にする必要がある。

        Args:
            user_agent (str, optional): User Agentを指定する. Defaults to None.
            browser (str, optional): Seleniumで使用するBrowserを指定する([chrome, firefox]). Defaults to None.
        """

        if user_agent is None:
            # seleniumが有効になっている場合、そのままSeleniumで利用するブラウザのUAを使用する
            if self.USE_SELENIUM:
                user_agent = ''
            else:
                try:
                    ua = UserAgent(verify_ssl=False, use_cache_server=True)
                    if user_agent is None:
                        if browser is None:
                            user_agent = ua.firefox

                        elif browser == 'chrome':
                            user_agent = ua.chrome

                        elif browser == 'firefox':
                            user_agent = ua.chrome

                except Exception:
                    user_agent = 'Mozilla/5.0 (Linux; Android 10; SM-A205U) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.131 Mobile Safari/537.36.'

        self.USER_AGENT = user_agent

    # seleniumを有効にする
    #   - splashより優先
    #   - host, browserは、指定がない場合はそれぞれデフォルト設定(hostは指定なし、browserはchrome)での動作
    #   - browserは `chrome` or `firefox` のみ受け付ける
    def set_selenium(self, uri: str = None, browser: str = None):  # type: ignore
        """set_selenium

        検索時にSelenium経由で通信を行う.
        他のHeadless Browserと比較して最優先(Splash等が有効でもこちらが優先される).

        Args:
            uri (str, optional): APIのURIを指定(localhost:4444). Defaults to None.
            browser (str, optional): 使用するブラウザを指定([chrome, firefox]). Defaults to None.
        """

        # 入力値検証(browser: chrome or firefox)
        if browser is None:
            browser = 'chrome'

        # USE_SELENIUM to True
        self.USE_SELENIUM = True
        self.SELENIUM_URI = uri
        self.SELENIUM_BROWSER = browser

    # proxyの設定を受け付ける
    def set_proxy(self, proxy: str):
        """set_proxy

        検索時に使用するProxyを指定する(uri指定)

        Args:
            proxy (str): ProxyのURIを指定する(socks5://localhost:11080, http://hogehoge:8080)
        """

        self.PROXY = proxy

    # splash urlの値を受け付ける
    def set_splash(self, splash_url: str):
        """set_splash

        検索時にSplashを有効にする.
        (Seleniumと同時に有効化されている場合、Seleniumを優先する)

        Args:
            splash_url (str): Splashのアクセス先URIを指定する(ex: `localhost:8050`)
        """

        self.USE_SPLASH = True
        self.SPLASH_URI = splash_url

    # common.Messageを受け付ける
    def set_messages(self, message: Message):
        self.MESSAGE = message

    # sslのチェックを無効にする
    def set_ignore_ssl(self, verify: bool):
        self.IGNORE_SSL_VERIFY = verify

    # cookieをcookiefileから取得する
    def read_cookies(self):
        """read_cookies

        `self.COOKIE_FILE` からcookieを読み込む.
        現時点ではSeleniumでのみ動作.
        """

        # cookieファイルが存在しない場合、空ファイルで作成する
        exist_cookie_file = os.path.isfile(self.COOKIE_FILE)
        if not exist_cookie_file:
            cookie_file = open(self.COOKIE_FILE, 'w')
            cookie_file.write('')
            cookie_file.close()

        # cookieファイルのサイズを取得
        file_size = os.path.getsize(self.COOKIE_FILE)

        # cookieファイルのサイズが0以上の場合
        if file_size > 0:
            # cookie fileからcookieの取得
            cookies = pickle.load(open(self.COOKIE_FILE, "rb"))

            # seleniumを使う場合
            if self.USE_SELENIUM:
                # 事前アクセスが必要になるため、検索対象ドメインのTOPページにアクセスしておく
                self.driver.get(self.ENGINE_TOP_URL)  # type: ignore

                # cookieを設定していく
                for cookie in cookies:
                    try:
                        self.driver.add_cookie(cookie)
                    except Exception:
                        pass

            # splashを使う場合
            elif self.USE_SPLASH:
                # NOTE: 動作しないためコメントアウト
                # TODO: 確認して修正
                # self.session.cookies.update(cookies)
                None

            # requestを使う場合
            else:
                # NOTE: 動作しないためコメントアウト
                # TODO: 確認して修正
                # self.session.cookies.update(cookies)
                None

    # cookieをcookiefileに書き込む
    def write_cookies(self):
        """write_cookies

        cookiesを `self.COOKIE_FILE` に書き込む.

        """

        cookies = None

        # seleniumを使う場合
        if self.USE_SELENIUM:
            cookies = self.driver.get_cookies()

        # splashを使う場合
        elif self.USE_SPLASH:
            cookies = self.session.cookies

        # requestを使う場合
        else:
            cookies = self.session.cookies

        # cookieを書き込み
        with open(self.COOKIE_FILE, 'wb') as f:
            pickle.dump(cookies, f)

    # seleniumのOptionsを作成
    def create_selenium_options(self):
        """create_selenium_options

        Seleniumのoptionsを生成して返す.

        Returns:
            Options: 指定されたブラウザに応じたSeleniumのOptionsを返す.
        """

        # browser別の処理
        if self.SELENIUM_BROWSER == 'chrome':
            options = ChromeOptions()

            # set ssl verify
            if not self.IGNORE_SSL_VERIFY:
                options.add_argument('ignore-certificate-errors')

        elif self.SELENIUM_BROWSER == 'firefox':
            options = FirefoxOptions()

        # set headless option
        if not self.IS_DISABLE_HEADLESS:
            options.add_argument('--headless')

        # set user_agent option
        if self.USER_AGENT != '':
            options.add_argument('--user-agent=%s' % self.USER_AGENT)

        return options

    # selenium driverの作成
    def create_selenium_driver(self):
        """create_selenium_driver

        Seleniumで使用するDriverを作成する関数.
        Optionsもこの関数で作成する.
        """

        # optionsを取得する
        options = self.create_selenium_options()

        # browserに応じてdriverを作成していく
        if self.SELENIUM_BROWSER == 'chrome':
            # proxyを追加
            if self.PROXY != '':
                options.add_argument('--proxy-server=%s' % self.PROXY)

            try:
                chromedriver_autoinstaller.install()
            except Exception:
                pass

            self.driver = Chrome(options=options)

        elif self.SELENIUM_BROWSER == 'firefox':
            # profileを作成する
            profile = webdriver.FirefoxProfile()
            profile.set_preference('devtools.jsonview.enabled', False)
            profile.set_preference('plain_text.wrap_long_lines', False)
            profile.set_preference('view_source.wrap_long_lines', False)

            # proxyを追加
            if self.PROXY != '':
                # self.PROXYをパース処理する
                parsed_uri = parse.urlparse(self.PROXY)

                # socks5
                if parsed_uri.scheme == "socks5":
                    # Proxy設定を追加
                    profile.set_preference(
                        'network.proxy.type', 1)
                    profile.set_preference('network.proxy.socks_version', 5)
                    profile.set_preference(
                        'network.proxy.socks', parsed_uri.hostname)
                    profile.set_preference(
                        'network.proxy.socks_port', parsed_uri.port)
                    profile.set_preference('network.proxy.no_proxies_on', '')
                    profile.set_preference(
                        'network.proxy.socks_remote_dns', True)
                    profile.update_preferences()
                elif parsed_uri.scheme == "socks4":
                    # Proxy設定を追加
                    profile.set_preference(
                        'network.proxy.type', 1)
                    profile.set_preference('network.proxy.socks_version', 4)
                    profile.set_preference(
                        'network.proxy.socks', parsed_uri.hostname)
                    profile.set_preference(
                        'network.proxy.socks_port', parsed_uri.port)
                    profile.set_preference('network.proxy.no_proxies_on', '')
                    profile.set_preference(
                        'network.proxy.socks_remote_dns', True)
                    profile.update_preferences()

            # set ssl verify(firefoxの場合はprofileで処理するのでこちらに記述する)
            if not self.IGNORE_SSL_VERIFY:
                profile.accept_untrusted_certs = True

            try:
                geckodriver_autoinstaller.install()
            except Exception:
                pass
            self.driver = Firefox(options=options, firefox_profile=profile)

        # NOTE:
        #   User Agentを確認する場合、↓の処理で実施可能(Chrome/Firefoxともに)。
        # ```python
        # user_agent = self.driver.execute_script("return navigator.userAgent")
        # print(user_agent)
        # ```

        return

    # selenium経由でリクエストを送信する
    def request_selenium(self, url: str, method='GET', data=None):
        """[summary]

        Selenium経由でGETリクエストを投げて、その結果をhtml(文字列)で返す.

        Args:
            url (str):    リクエストを投げるurl.
            method (str): リクエストメソッド.
            data (str):   POSTメソッド時に利用するdata.

        Returns:
            str: htmlの文字列.
        """

        if method == 'GET':
            response = self.driver.get(url)

            # wait all elements
            WebDriverWait(self.driver, 15).until(
                EC.presence_of_all_elements_located)

            # wait 5 seconds(wait DOM)
            if self.NAME in ('Bing', 'Baidu', 'DuckDuckGo'):  # type: ignore
                self.driver.implicitly_wait(20)

            # get result
            result = self.driver.page_source

        elif method == 'POST':
            response = self.driver.request('POST', url, data=data)

            # wait all elements
            WebDriverWait(self.driver, 15).until(
                EC.presence_of_all_elements_located)

            # wait 5 seconds(wait DOM)
            if self.NAME in ('Bing', 'Baidu', 'DuckDuckGo'):  # type: ignore
                self.driver.implicitly_wait(20)

            # get result
            result = response.text

        return result

    # splash経由でのリクエストを送信する
    def request_splash(self, url: str, method='GET', data=None):
        """request_splash

        Splash経由でGETリクエストを投げて、その結果をhtml(文字列)で返す.

        Args:
            url (str):    リクエストを投げるurl.
            method (str): リクエストメソッド.
            data (str):   POSTメソッド時に利用するdata.

        Returns:
            str: htmlの文字列.
        """

        # urlを生成する
        splash_url = 'http://' + self.SPLASH_URI + '/render.html'

        # param
        params = {
            'url': url
        }

        # Proxy指定をする場合
        if self.PROXY != '':
            params['proxy'] = self.PROXY

        # リクエストを投げてレスポンスを取得する
        if method == 'GET':
            result = self.session.get(splash_url, params=params).text

        # NOTE: Googleの画像検索のPOSTがSplashではレンダリングできないので、特例対応でrequestsを使用する.
        # TODO: Splashでもレンダリングできるようになったら書き換える.
        elif method == 'POST' and self.NAME == 'Google' and self.IMAGE_URL in url:  # type: ignore
            # create session
            session = requests.session()

            # proxyを設定
            if self.PROXY != '':
                proxies = {
                    'http': self.PROXY,
                    'https': self.PROXY
                }
                session.proxies = proxies

            # user-agentを設定
            if self.USER_AGENT != '':
                session.headers.update(
                    {
                        'User-Agent': self.USER_AGENT,
                        'Accept-Language': 'ja,en-US;q=0.7,en;q=0.3'
                    }
                )

            result = session.post(url, data=data).text

        elif method == 'POST':
            headers = {'Content-Type': 'application/json'}
            params['http_method'] = 'POST'
            params['body'] = parse.urlencode(data)  # type: ignore

            result = self.session.post(
                splash_url,
                headers=headers,
                json=params
            ).text

        return result

    # seleniumやsplushなどのヘッドレスブラウザ、request.sessionの作成・設定、cookieの読み込みを行う
    def create_session(self):
        """create_session

        指定された接続方式(Seleniumなどのヘッドレスブラウザの有無)に応じて、driverやsessionを作成する.
        cookiesの読み込みやproxyの設定が必要な場合、この関数内で処理を行う.
        """

        # seleniumを使う場合
        if self.USE_SELENIUM:
            self.create_selenium_driver()

        # splashを使う場合
        elif self.USE_SPLASH:
            # create session
            self.session = requests.session()

            # user-agentを設定
            if self.USER_AGENT != '':
                self.session.headers.update(
                    {
                        'User-Agent': self.USER_AGENT,
                        'Accept-Language': 'ja,en-US;q=0.7,en;q=0.3'
                    }
                )

        # requestを使う場合
        else:
            # create session
            self.session = requests.session()

            # リダイレクトの上限を60にしておく(baidu対策)
            self.session.max_redirects = 60

            # proxyを設定
            if self.PROXY != '':
                proxies = {
                    'http': self.PROXY,
                    'https': self.PROXY
                }
                self.session.proxies = proxies

            # user-agentを設定
            if self.USER_AGENT != '':
                self.session.headers.update(
                    {
                        'User-Agent': self.USER_AGENT,
                        'Accept-Language': 'ja,en-US;q=0.7,en;q=0.3'
                    }
                )

        # cookiefileが指定されている場合、読み込みを行う
        if self.COOKIE_FILE != '':
            self.read_cookies()

        return

    # sessionをcloseする
    def close_session(self):
        if self.USE_SELENIUM:
            self.driver.quit()
        else:
            self.session.close()

    # リクエストを投げてhtmlを取得する(selenium/splash/requestで分岐してリクエストを投げるwrapperとして動作させる)
    def get_result(self, url: str, method='GET', data=None):
        """get_result

        接続方式に応じて、urlへGETリクエストを投げてhtmlを文字列で返す関数.

        Args:
            url (str):    リクエストを投げるurl.
            method (str): リクエストメソッド.
            data (str):   POSTメソッド時に利用するdata.

        Returns:
            str: htmlの文字列.
        """

        # 優先度1: Selenium経由でのアクセス
        if self.USE_SELENIUM:
            result = self.request_selenium(url, method=method, data=data)

        # 優先度2: Splash経由でのアクセス(Seleniumが有効になってない場合はこちら)
        elif self.USE_SPLASH:
            # create splash url
            result = self.request_splash(url, method=method, data=data)

        # 優先度3: request.sessionからのリクエスト(SeleniumもSplashも有効でない場合)
        else:
            if method == 'GET':
                result = self.session.get(
                    url, verify=self.IGNORE_SSL_VERIFY).text
            elif method == 'POST':
                result = self.session.post(
                    url, verify=self.IGNORE_SSL_VERIFY, data=data).text

        return result

    # 検索用のurlを生成
    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.
        各検索エンジンで上書きする用の関数.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: method
            dict: 検索用url
            dict: data
        """

        result = {}
        return 'GET', result, None

    # テキスト、画像検索の結果からlinksを取得するための集約function
    def get_links(self, source_url, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            url  (str): 解析する検索結果のurl.
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]`)
        """

        # BeautifulSoupでの解析を実施
        soup = BeautifulSoup(html, 'lxml')

        if type == 'text':
            # link, titleの組み合わせを取得する
            elinks, etitles, etexts = self.get_text_links(soup)

            # before processing elists
            self.MESSAGE.print_text(
                ','.join(elinks),  # type: ignore
                header=self.MESSAGE.HEADER + ': ' + Color.BLUE +
                '[BeforeProcessing elinks]' + Color.END,
                separator=" :",
                mode="debug",
            )

            # before processing etitles
            self.MESSAGE.print_text(
                ','.join(etitles),
                header=self.MESSAGE.HEADER + ': ' +
                Color.BLUE + '[BeforeProcessing etitles]' + Color.END,
                separator=" :",
                mode="debug",
            )

            # 加工処理を行う関数に渡す(各エンジンで独自対応)
            elinks, etitles, etexts = self.processings_elist(
                elinks, etitles, etexts)

            # after processing elists
            self.MESSAGE.print_text(
                ','.join(elinks),  # type: ignore
                header=self.MESSAGE.HEADER + ': ' +
                Color.GREEN + '[AfterProcessing elinks]' + Color.END,
                separator=" :",
                mode="debug",
            )

            # after processing etitles
            self.MESSAGE.print_text(
                ','.join(etitles),
                header=self.MESSAGE.HEADER + ': ' +
                Color.GREEN + '[AfterProcessing etitles]' + Color.END,
                separator=" :",
                mode="debug",
            )

            # dictに加工してリスト化する
            # [{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]
            links = self.create_text_links(source_url, elinks, etitles, etexts)

            return links

        elif type == 'image':
            links = self.get_image_links(soup)

            return links

    # テキスト検索ページの検索結果(links([{link: ..., title: ...},...]))を生成するfunction
    def get_text_links(self, soup: BeautifulSoup):
        """get_text_links

        BeautifulSoupからテキスト検索ページを解析して結果を返す関数.

        Args:
            soup (BeautifulSoup): 解析するBeautifulSoupオブジェクト.

        Returns:
            list: linkの検索結果([xxx,xxx,xxx...])
            list: titleの検索結果([xxx,xxx,xxx...])
            list: textの検索結果([xxx,xxx,xxx...])
        """
        # linkのurlを取得する
        elements = soup.select(self.SOUP_SELECT_URL)
        elinks = [e['href'] for e in elements]

        # linkのtitleを取得する
        elements = soup.select(self.SOUP_SELECT_TITLE)
        etitles = [e.text for e in elements]

        # linkのtextを取得する
        elements = soup.select(self.SOUP_SELECT_TEXT)
        etext = [e.text for e in elements]

        return elinks, etitles, etext

    # 画像検索ページの検索結果(links(list()))を生成するfunction
    def get_image_links(self, soup: BeautifulSoup):
        """get_image_links
        BeautifulSoupから画像検索ページを解析して結果を返す関数.
        (実際の処理は各検索エンジンごとの関数で実施).

        Args:
            soup (BeautifulSoup): 解析するBeautifulSoupオブジェクト.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]`)
        """

        links = []

        return links

    # elist, etitle生成時の追加編集処理用function
    def processings_elist(self, elinks, etitles, etexts: list):
        """processings_elist

        self.get_links 内で、取得直後のelinks, etitlesに加工を加えるための関数.
        必要に応じて各検索エンジンのClassで上書きする.

        Args:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列

        Returns:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列
        """

        return elinks, etitles, etexts

    # テキスト検索の1ページごとの検索結果から、links(links([{link: ..., title: ...},...]))を生成するfunction
    def create_text_links(self, source_url: str, elinks, etitles, etext: list):
        """create_text_links

        elinks, etitlesからlinks(get_linksのデータ)を返す関数.

        Args:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etext (list): etext(検索結果のテキスト)の配列

        Returns:
            list: 検索結果(`[{'title': 'title...', 'url': 'https://hogehoge....', 'text': 'hogehoge fugafuga...'}, {...}]`)を返す。
        """

        links = list()
        n = 0
        before_link = ""
        for link in elinks:
            d = dict()
            d['link'] = link

            # etitle(urlのtitle)をdictに追加する
            if len(etitles) > n:
                d['title'] = etitles[n]

            # etext(urlに対応する検索結果のテキスト文)をdictに追加する
            if len(etext) > n:
                d['text'] = etext[n]

            # 検索元urlをdictに追加する
            d['source_url'] = source_url

            if before_link != link:
                links.append(d)

            before_link = link
            n += 1

        return links

    # サジェスト取得用のurlを生成
    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.
        各検索エンジンで上書きする用の関数.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """

        result = {}
        return result

    # サジェストの取得
    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.
        実際の処理は各検索エンジンClassで上書きする.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """
        result = {}
        return result

    # ReCaptcha画面かどうかを識別するための関数
    def check_recaptcha(self, html: str):
        """[summary]

        `self.SOUP_RECAPTCHA_TAG` を元に、htmlがReCaptcha画面かどうかを識別する.

        Args:
            html (str): 識別するページのhtml

        Returns:
            bool: ReCaptcha画面かどうか(ReCaptcha画面の場合はTrue)
        """

        result = False

        # BeautifulSoupでの識別を実施
        soup = BeautifulSoup(html, 'lxml')

        # 要素が存在するかを確認
        if self.SOUP_RECAPTCHA_TAG != '':
            elements = soup.select(self.SOUP_RECAPTCHA_TAG)

            # 要素のチェック
            if len(elements) > 0:
                result = True

        return result

    # ReCaptchaをBypassする処理(wrapper)
    def bypass_recaptcha(self, url: str, html: str):
        """bypass_recaptcha

        ReCaptcha画面をBypassするための関数.
        実際の処理は Selenium/Splash 各ブラウザに応じて処理させる関数に行わせる.

        Args:
            url (str): ReCaptcha画面が表示されてしまったリクエストのurl
            html (str): ReCaptcha画面のhtml

        Returns:
            str: ReCaptchaを突破後のurlのhtml
        """

        # seleniumを使う場合
        if self.USE_SELENIUM:
            html = self.bypass_recaptcha_selenium(url, html)

        # splashを使う場合
        elif self.USE_SPLASH:
            html = self.bypass_recaptcha_splash(url, html)

        return html

    # ReCaptchaをSeleniumでBypassする処理
    def bypass_recaptcha_selenium(self, url: str, html: str):
        """bypass_recaptcha_selenium

        SeleniumでReCaptchaを突破する関数.
        実際の処理は各検索エンジンのClassで実装.

        Args:
            url (str): ReCaptcha画面が表示されてしまったリクエストのurl
            html (str): ReCaptcha画面のhtml

        Returns:
            str: ReCaptchaを突破後のurlのhtml
        """

        return html

    # ReCaptchaをSplashでBypassする処理
    def bypass_recaptcha_splash(self, url: str, html: str):
        """bypass_recaptcha_splash

        SplashでReCaptchaを突破する関数.
        実際の処理は各検索エンジンのClassで実装.

        Args:
            url (str): ReCaptcha画面が表示されてしまったリクエストのurl
            html (str): ReCaptcha画面のhtml

        Returns:
            str: ReCaptchaを突破後のurlのhtml
        """

        return html



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""engine_baidu
    * Baidu用の検索用Classを持つモジュール.
"""

import requests
import json
import asyncio
import sys

from urllib import parse
from bs4 import BeautifulSoup

from .engine_common import CommonEngine
from .common import Color


class Baidu(CommonEngine):
    """Baidu

    Baidu用の検索エンジン用Class.

    Note:
        検索結果に直接urlが記載されておらず、リンクを踏んで移動先のurlを取得する必要がある。
        そのため、検索結果を取得してからパラレルで検索結果urlからリンク先urlを取得している。
        なお、その際のリクエストはSelenium/Splashを使用している場合でもrequestsを使っている。
    """

    def __init__(self):
        # CommonEngineの処理を呼出し
        super().__init__()

        self.NAME = 'Baidu'
        self.COLOR = Color.RED
        self.COLOR_NAME = self.COLOR + self.NAME + Color.END

        # リクエスト先のURLを指定
        self.ENGINE_TOP_URL = 'https://www.baidu.com/'
        self.SEARCH_URL = 'https://www.baidu.com/s'
        self.IMAGE_URL = 'https://image.baidu.com/search/acjson'
        self.SUGGEST_URL = 'https://www.baidu.com/sugrec'

    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: 検索用url
        """

        if type == 'text':
            # 1ページごとの表示件数
            view_num = 50

            # 検索urlを指定
            search_url = self.SEARCH_URL

            # 検索パラメータの設定
            url_param = {
                'wd': keyword,  # 検索キーワード
                'rn': view_num,     # 1ページごとの表示件数
                'filter': '0',  # aaa
                'ia': 'web',    #
                'pn': ''        # 開始位置
            }

        elif type == 'image':
            # 1ページごとの表示件数
            view_num = 30

            # example:
            #   'https://image.baidu.com/search/acjson?tn=resultjson_com&logid=10696586825489113064&ipn=rj&ct=201326592&is=&fp=result&queryWord=poop&cl=2&lm=-1&ie=utf-8&oe=utf-8&adpicid=&st=&z=&ic=&hd=&latest=&copyright=&word=poop&s=&se=&tab=&width=&height=&face=&istype=&qc=&nc=1&fr=&expermode=&force=&pn=30&rn=30&gsm=1e&1617708591950='
            #   'https://image.baidu.com/search/acjson?tn=resultjson_com&logid=11967476791890431299&ipn=rj&ct=201326592&is=&fp=result&queryWord=poop&cl=2&lm=-1&ie=utf-8&oe=utf-8&adpicid=&st=&z=&ic=&hd=&latest=&copyright=&word=poop&s=&se=&tab=&width=&height=&face=&istype=&qc=&nc=1&fr=&expermode=&nojc=&pn=60&rn=30&gsm=3c&1629026924429='

            # 検索urlを指定
            search_url = self.IMAGE_URL

            # 検索パラメータの設定
            url_param = {
                'tn': 'resultjson_com',
                'fp': 'result',
                'queryWord': keyword,
                'word': keyword,
                'logid': '12399428100030957064',
                'ipn': 'rj',
                'ct': '201326592',
                'lm': '-1',
                'cl': 2,
                'ie': 'utf-8',
                'nc': 1,
                'pn': 0,  # 開始位置
                'rn': view_num,
                'gsm': '3c',
            }

        page = 0
        while True:
            # parameterにページを開始する番号を指定
            url_param['pn'] = str(page * view_num)
            params = parse.urlencode(url_param)

            target_url = search_url + '?' + params

            yield 'GET', target_url, None

            page += 1

    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """

        url_param = {
            'wd': keyword,  # 検索キーワード
            'prod': 'pc'   #
        }

        params = parse.urlencode(url_param)
        url = self.SUGGEST_URL + '?' + params

        return url

    def get_links(self, url: str, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'url': 'https://hogehoge....'}, {...}]`)
        """

        links = []

        if type == 'text':
            # Splash経由で通信している場合
            self.SOUP_SELECT_URL = '.tts-title > a'
            self.SOUP_SELECT_TITLE = '.tts-title > a'
            self.SOUP_SELECT_TEXT = '.c-gap-top-small > span'

            # CommonEngineの処理を呼び出す
            links = super().get_links(url, html, type)

        elif type == 'image':
            # unicode escape
            # html = html.encode().decode("unicode-escape")
            html = html.replace("\\'", "'")

            # seleniumを使用している場合、htmlで返ってくるためjson要素のみを抽出する
            if self.USE_SELENIUM:
                html_text = ""
                soup = BeautifulSoup(html, "lxml")

                for text in soup.find_all(text=True):
                    if text.strip():
                        html_text += text

                html = html_text

            # json load
            try:
                json_data = json.loads(html, strict=False)
            except Exception as e:
                print(e, file=sys.stderr)
                return links

            if 'data' in json_data:
                data = json_data['data']

                for d in data:
                    if 'replaceUrl' in d:
                        result = dict()

                        # 画像ファイルのurlをパラメータに持つvalueを取得する
                        replace_url = d['replaceUrl'][0]['ObjURL']
                        replace_url = replace_url.replace(
                            'image_search/', 'image_search/?')

                        # url valueをparse
                        replace_url_query = parse.urlparse(replace_url).query

                        # パラメータを取得
                        replace_url_query_dict = parse.parse_qs(
                            replace_url_query)

                        if 'src' not in replace_url_query_dict:
                            continue

                        # 画像urlを取得
                        result['link'] = replace_url_query_dict['src'][0]

                        if 'fromPageTitle' in d:
                            result['title'] = d['fromPageTitle']

                        links.append(result)

        return links

    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """

        try:
            data = json.loads(html)
        except Exception:
            soup = BeautifulSoup(html, "lxml")
            json_data = soup.select_one('html > body')
            data = json.loads(json_data.text)

        if 'g' in data:
            suggests[char if char == '' else char[-1]
                     ] = [e['q']
                          for e in data['g']]
        return suggests

    def processings_elist(self, elinks, etitles, etexts: list):
        """processings_elist

        self.get_links 内で、取得直後のelinks, etitlesに加工を加えるための関数.
        requestsを用いて、リダイレクトリンクから遷移先urlを取得していく.

        Args:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列

        Returns:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
            etexts (list): etexts(検索結果のtext)の配列
        """

        # 通常のスクレイピングとは別にセッションを作成
        session = requests.session()

        # pool sizeを調整
        adapter = requests.adapters.HTTPAdapter(
            pool_connections=100, pool_maxsize=100)
        session.mount('http://', adapter)

        # proxyを設定
        if self.PROXY != '':
            proxies = {
                'http': self.PROXY,
                'https': self.PROXY
            }
            session.proxies = proxies

        # user-agentを設定
        if self.USER_AGENT != '':
            session.headers.update(
                {
                    'User-Agent': self.USER_AGENT,
                    'Accept-Language': 'ja,en-US;q=0.7,en;q=0.3'
                }
            )

        # asyncio loopを作成
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        # リダイレクト先のurlに置き換え
        elinks = loop.run_until_complete(
            resolv_links(loop, session, elinks))
        loop.close()

        return elinks, etitles, etexts


async def resolv_links(loop: asyncio.AbstractEventLoop, session: requests.Session, links: list):
    """resolv_links

    リダイレクト先のurlをパラレルで取得する(Baiduで使用)

    Args:
        loop (asyncio.AbstractEventLoop): loop
        session (requests.Session): 使用するSession
        links (list): リダイレクト先を取得するurlのリスト

    Returns:
        data (list): リダイレクト先を取得したurlのリスト
    """

    async def req(session: requests.Session, url: str):
        task = await loop.run_in_executor(None, resolv_url, session, url)
        return task

    tasks = []
    for link in links:
        task = req(session, link)
        tasks.append(task)

    data = await asyncio.gather(*tasks)

    return data


def resolv_url(session: requests.Session, url: str):
    """resolv_url

    リダイレクト先のurlを取得する(Baiduで使用)

    Args:
        session (request.Session): リダイレクト先を取得する際に使用するSession
        url (str): リダイレクト先を取得するurl
    Returns:
        url (str): リダイレクト先のurl
    """
    while True:
        try:
            res_header = session.head(url, allow_redirects=False).headers
        except requests.RequestException:
            continue
        except ConnectionError:
            continue
        else:
            url = res_header['Location']
            break

    return url



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================

from .sub_commands import run_subcommand
from .engine import ENGINES
from . import messages

from pkg_resources import get_distribution
from datetime import datetime

import copy
import argparse

# TODO: returnではなくyieldに切り替えて、返り値をgeneratorにすることである程度途中状態でも状況を見れるような仕組みとする


# version (setup.pyから取得してくる)
__version__ = get_distribution('pydork').version


# main
def main():
    # parserの作成
    parser = argparse.ArgumentParser(
        description=messages.description)
    subparsers = parser.add_subparsers()

    # ENGINESに`all`を追加
    engines_list = copy.deepcopy(ENGINES)
    engines_list.append('all')

    # サブコマンド共通の引数
    common_args_map = [
        {
            "args": ["query"],
            "action": "store",
            "type": str,
            "nargs": "?",
            "default": "",
            "help": messages.help_message_query,
        },
        {
            "args": ["-f", "--file"],
            "action": "store",
            "type": str,
            "default": "",
            "help": messages.help_message_op_file,
        },
        {
            "args": ["-F", "--template_file"],
            "action": "store",
            "type": str,
            "default": "",
            "help": messages.help_message_op_template_file,
        },
        {
            "args": ["-V", "--template_variable"],
            "action": "store",
            "type": str,
            "default": "",
            "help": messages.help_message_op_template_variable,
        },
        {
            "args": ["-t", "--search_type"],
            "default": ["google"],
            "choices": engines_list,
            "nargs": "+",
            "type": str,
            "help": messages.help_message_op_search_type,
        },
        {
            "args": ["-l", "--lang"],
            "default": "ja",
            "choices": ["ja", "en"],
            "type": str,
            "help": messages.help_message_op_lang,
        },
        {
            "args": ["-c", "--country"],
            "default": "JP",
            "choices": ["JP", "US"],
            "type": str,
            "help": messages.help_message_op_country,
        },
        {
            "args": ["-P", "--proxy"],
            "default": "",
            "type": str,
            "help": messages.help_message_op_proxy_server,
        },
        {
            "args": ["-j", "--json"],
            "action": "store_true",
            "help": messages.help_message_op_json,
        },
        {
            "args": ["-k", "--insecure"],
            "action": "store_true",
            "help": messages.help_message_op_insecure,
        },
        {
            "args": ["-s", "--selenium"],
            "action": "store_true",
            "help": messages.help_message_op_selenium,
        },
        {
            "args": ["-S", "--splash"],
            "action": "store_true",
            "help": messages.help_message_op_splash,
        },
        {
            "args": ["-b", "--browser-endpoint"],
            "default": "",
            "type": str,
            "help": messages.help_message_op_browser_endpoint,
        },
        {
            "args": ["-B", "--browser"],
            "default": "firefox",
            "choices": ["chrome", "firefox"],
            "type": str,
            "help": messages.help_message_op_browser,
        },
        {
            "args": ["--color"],
            "default": "auto",
            "choices": ["auto", "none", "always"],
            "type": str,
            "help": messages.help_message_op_color,
        },
        {
            "args": ["--cookies"],
            "default": "~/.pydork_cookies",
            "type": str,
            "help": messages.help_message_op_cookies_dir,
        },
        {
            "args": ["--delete-cookies"],
            "action": "store_true",
            "help": messages.help_message_op_delete_cookies,
        },
    ]

    # サブコマンド `search` の引数
    search_args_map = [
        {
            "args": ["-T", "--title"],
            "action": "store_true",
            "help": messages.help_message_op_title,
        },
        {
            "args": ["-0", "--nullchar"],
            "action": "store_true",
            "help": messages.help_message_op_null_char,
        },
        {
            "args": ["-n", "--num"],
            "default": 300,
            "type": int,
            "help": messages.help_message_op_num,
        },
        {
            "args": ["--start"],
            "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
            "help": messages.help_message_op_start,
        },
        {
            "args": ["--end"],
            "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
            "help": messages.help_message_op_end,
        },
        {
            "args": ["--debug"],
            "action": "store_true",
            "help": messages.help_message_op_debug,
        },
        {
            "args": ["--disable-headless"],
            "action": "store_true",
            "help": messages.help_message_op_disable_headless,
        },
    ]
    search_args_map.extend(copy.deepcopy(common_args_map))

    # サブコマンド `image` の引数
    image_args_map = [
        {
            "args": ["-T", "--title"],
            "action": "store_true",
            "help": messages.help_message_op_title,
        },
        {
            "args": ["-p", "--pagelink"],
            "action": "store_true",
            "help": messages.help_message_op_image_pagelink,
        },
        {
            "args": ["-0", "--nullchar"],
            "action": "store_true",
            "help": messages.help_message_op_null_char,
        },
        {
            "args": ["-n", "--num"],
            "default": 300,
            "type": int,
            "help": messages.help_message_op_num,
        },
        # {
        #     "args": ["--start"],
        #     "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
        #     "help": messages.help_message_op_start,
        # },
        # {
        #     "args": ["--end"],
        #     "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
        #     "help": messages.help_message_op_end,
        # },
        {
            "args": ["--debug"],
            "action": "store_true",
            "help": messages.help_message_op_debug,
        },
        {
            "args": ["--disable-headless"],
            "action": "store_true",
            "help": messages.help_message_op_disable_headless,
        },
    ]
    image_args_map.extend(copy.deepcopy(common_args_map))

    # サブコマンド `suggest` の引数
    suggest_args_map = [
        {
            "args": ["--jap"],
            "action": "store_true",
            "help": messages.help_message_op_suggest_jap
        },
        {
            "args": ["--alph"],
            "action": "store_true",
            "help": messages.help_message_op_suggest_alph
        },
        {
            "args": ["--num"],
            "action": "store_true",
            "help": messages.help_message_op_suggest_num
        },
    ]
    suggest_args_map.extend(copy.deepcopy(common_args_map))

    # search
    # ----------
    parser_search = subparsers.add_parser(
        'search',
        help='search mode. see `search -h`'
    )

    # add_argument
    for element in search_args_map:
        args = element['args']
        element.pop('args')
        parser_search.add_argument(*args, **element)

    # set parser_search
    parser_search.set_defaults(handler=run_subcommand, subcommand="search")

    # image
    # ----------
    parser_image = subparsers.add_parser(
        'image',
        help='search mode. see `search -h`'
    )

    # add_argument
    for element in image_args_map:
        args = element['args']
        element.pop('args')
        parser_image.add_argument(*args, **element)

    # set parser_image
    parser_image.set_defaults(handler=run_subcommand, subcommand="image")

    # suggest
    # ----------
    parser_suggest = subparsers.add_parser(
        'suggest',
        help='suggest mode. see `suggest -h`'
    )

    # add_argument
    for element in suggest_args_map:
        args = element['args']
        element.pop('args')
        parser_suggest.add_argument(*args, **element)

    parser_suggest.set_defaults(handler=run_subcommand, subcommand="suggest")

    # --version(-v)オプションのparser定義
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version='%(prog)s version:{version}'.format(version=__version__)
    )

    args = parser.parse_args()
    if hasattr(args, 'handler'):
        args.handler(args.subcommand, args)
    else:
        # 未知のサブコマンドの場合はヘルプを表示
        parser.print_help()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


"""test_engine_google
    * SearchEngine Classのテストコード.
    * 各検索エンジンの動作テストを行う
"""

# TODO: splash/selenium経由での通信のテストも追加する(dockerでのコンテナ環境が前提になると思われる)


import unittest

from .engine import SearchEngine

# 変数
SEARCH_TEXT = 'Linux'


class SearchEngineTestCaseWithSelenium(unittest.TestCase):
    def setUp(self):
        """setUp

        テストメソッド実行前処理.
        """
        # SearchEngine
        self.search_engine = SearchEngine()

        print("setUp!!")

    def tearDown(self):
        """tearDown

        テストメソッド実行後処理
        """

        print("tearDown!!")

    def common_settings(self):
        # command modeを有効化
        self.search_engine.set_is_command(True)

        # debug modeを有効化
        self.search_engine.set_is_debug(True)

        # seleniumを有効化
        self.search_engine.set_selenium(None, 'chrome')

        # user agentを定義
        self.search_engine.set_user_agent()

    # ==========
    # Baidu
    # ==========
    def test_baidu_text_search(self):
        print('Test Baidu text search.')

        # 検索エンジンを指定(ここではBaiduを使用)
        self.search_engine.set('baidu')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, maximum=30)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_baidu_image_search(self):
        print('Test Baidu image search.')

        # 検索エンジンを指定(ここではBaiduを使用)
        self.search_engine.set('baidu')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, type='image', maximum=30)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_baidu_suggest(self):
        print('Test Baidu text suggest.')

        # 検索エンジンを指定(ここではBaiduを使用)
        self.search_engine.set('baidu')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_baidu_suggest_with_alph(self):
        print('Test Baidu text suggest with alph.')

        # 検索エンジンを指定(ここではBaiduを使用)
        self.search_engine.set('baidu')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, alph=True)

        self.assertNotEqual(0, len(data))

    def test_baidu_suggest_with_num(self):
        print('Test Baidu text suggest with num.')

        # 検索エンジンを指定(ここではBaiduを使用)
        self.search_engine.set('baidu')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, num=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    # ==========
    # Bing
    # ==========
    def test_bing_text_search(self):
        print('Test Bing text search.')

        # 検索エンジンを指定(ここではBingを使用)
        self.search_engine.set('bing')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, maximum=30)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_bing_image_search(self):
        print('Test Bing image search.')

        # 検索エンジンを指定(ここではBingを使用)
        self.search_engine.set('bing')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, type='image', maximum=30)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_bing_suggest(self):
        print('Test Bing text suggest.')

        # 検索エンジンを指定(ここではBingを使用)
        self.search_engine.set('bing')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_bing_suggest_with_jap(self):
        print('Test Bing text suggest with jap.')

        # 検索エンジンを指定(ここではBingを使用)
        self.search_engine.set('bing')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, jap=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_bing_suggest_with_alph(self):
        print('Test Bing text suggest with alph.')

        # 検索エンジンを指定(ここではBingを使用)
        self.search_engine.set('bing')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, alph=True)

        self.assertNotEqual(0, len(data))

    def test_bing_suggest_with_num(self):
        print('Test Bing text suggest with num.')

        # 検索エンジンを指定(ここではBingを使用)
        self.search_engine.set('bing')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, num=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    # ==========
    # DuckDuckGo
    # ==========
    def test_duckduckgo_text_search(self):
        print('Test DuckDuckGo text search.')

        # 検索エンジンを指定(ここではDuckDuckGoを使用)
        self.search_engine.set('duckduckgo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, maximum=30)

        print("{} count.".format(len(data)))
        self.assertEqual(30, len(data))

    def test_duckduckgo_image_search(self):
        print('Test DuckDuckGo image search.')

        # 検索エンジンを指定(ここではDuckDuckGoを使用)
        self.search_engine.set('duckduckgo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, type='image', maximum=30)

        print("{} count.".format(len(data)))
        self.assertEqual(30, len(data))

    def test_duckduckgo_suggest(self):
        print('Test DuckDuckGo text suggest.')

        # 検索エンジンを指定(ここではDuckDuckGoを使用)
        self.search_engine.set('duckduckgo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_duckduckgo_suggest_with_jap(self):
        print('Test DuckDuckGo text suggest with jap.')

        # 検索エンジンを指定(ここではDuckDuckGoを使用)
        self.search_engine.set('duckduckgo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, jap=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_duckduckgo_suggest_with_alph(self):
        print('Test DuckDuckGo text suggest with alph.')

        # 検索エンジンを指定(ここではDuckDuckGoを使用)
        self.search_engine.set('duckduckgo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, alph=True)

        self.assertNotEqual(0, len(data))

    def test_duckduckgo_suggest_with_num(self):
        print('Test DuckDuckGo text suggest with num.')

        # 検索エンジンを指定(ここではDuckDuckGoを使用)
        self.search_engine.set('duckduckgo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, num=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    # ==========
    # Google
    # ==========
    def test_google_text_search(self):
        print('Test Google text search.')

        # 検索エンジンを指定(ここではGoogleを使用)
        self.search_engine.set('google')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, maximum=30)

        print("{} count.".format(len(data)))
        self.assertEqual(30, len(data))

    def test_google_image_search(self):
        print('Test Google image search.')

        # 検索エンジンを指定(ここではGoogleを使用)
        self.search_engine.set('google')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, type='image', maximum=30)

        print("{} count.".format(len(data)))
        self.assertEqual(30, len(data))

    def test_google_suggest(self):
        print('Test Google text suggest.')

        # 検索エンジンを指定(ここではGoogleを使用)
        self.search_engine.set('google')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_google_suggest_with_jap(self):
        print('Test Google text suggest with jap.')

        # 検索エンジンを指定(ここではGoogleを使用)
        self.search_engine.set('google')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, jap=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_google_suggest_with_alph(self):
        print('Test Google text suggest with alph.')

        # 検索エンジンを指定(ここではGoogleを使用)
        self.search_engine.set('google')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, alph=True)

        self.assertNotEqual(0, len(data))

    def test_google_suggest_with_num(self):
        print('Test Google text suggest with num.')

        # 検索エンジンを指定(ここではGoogleを使用)
        self.search_engine.set('google')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, num=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    # ==========
    # Yahoo
    # ==========
    def test_yahoo_text_search(self):
        print('Test Yahoo text search.')

        # 検索エンジンを指定(ここではYahooを使用)
        self.search_engine.set('yahoo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, maximum=30)

        print("{} count.".format(len(data)))
        self.assertEqual(30, len(data))

    def test_yahoo_image_search(self):
        print('Test Yahoo image search.')

        # 検索エンジンを指定(ここではYahooを使用)
        self.search_engine.set('yahoo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.search(SEARCH_TEXT, type='image', maximum=30)

        print("{} count.".format(len(data)))
        self.assertEqual(30, len(data))

    def test_yahoo_suggest(self):
        print('Test Yahoo text suggest.')

        # 検索エンジンを指定(ここではYahooを使用)
        self.search_engine.set('yahoo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_yahoo_suggest_with_jap(self):
        print('Test Yahoo text suggest with jap.')

        # 検索エンジンを指定(ここではYahooを使用)
        self.search_engine.set('yahoo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, jap=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))

    def test_yahoo_suggest_with_alph(self):
        print('Test Yahoo text suggest with alph.')

        # 検索エンジンを指定(ここではYahooを使用)
        self.search_engine.set('yahoo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, alph=True)

        self.assertNotEqual(0, len(data))

    def test_yahoo_suggest_with_num(self):
        print('Test Yahoo text suggest with num.')

        # 検索エンジンを指定(ここではYahooを使用)
        self.search_engine.set('yahoo')

        # 共通系の中間前処理を実行
        self.common_settings()

        # 検索を実行
        data = self.search_engine.suggest(
            SEARCH_TEXT, num=True)

        print("{} count.".format(len(data)))
        self.assertNotEqual(0, len(data))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================

"""engine
    * ReCaptcha関連のClassを集約するモジュールファイル
"""

import json
import requests

from urllib import parse
from urllib.parse import urlparse
from bs4 import BeautifulSoup
from time import sleep

from .common import Color, Message


# 2CaptchaのAPIへPOSTするためのClass
class TwoCaptcha:
    """TwoCaptcha

    2CaptchaのAPIへPOSTし、ReCaptchaを突破するためのClass.

    Note:
        公式ライブラリ側でCookieのPOSTに対応していなかったため作成.


    """

    def __init__(self, apikey: str):
        """__init__

        Args:
            apikey (str): 2CaptchaのAPI Key.
        """

        # apiへリクエストを投げるためのsession
        self.session = requests.Session()

        # api_url
        self.api_in_url = 'https://2captcha.com/in.php'
        self.api_res_url = 'https://2captcha.com/res.php'

        # api_key
        self.api_key = apikey

        # proxy
        self.proxy = None
        self.user_agent = None

        # flag
        self.IS_DEBUG = False
        self.IS_COMMAND = False

        # Message
        self.MESSAGE = None

    def set_debug(self, is_debug: bool):
        """set_debug

        Args:
            is_debug (bool): debug modeが有効ならTrue
        """

        self.IS_DEBUG = is_debug

    def set_command(self, is_command: bool):
        """set_command

        Args:
            is_command (bool): command modeが有効ならTrue
        """

        self.IS_COMMAND = is_command

    def set_user_agent(self, user_agent: str):
        """set_user_agent

        Args:
            user_agent (str): 2Captchaに送るUser Agent
        """

        self.user_agent = user_agent

    def set_messages(self, message: Message):
        """set_message

        Args:
            message (Message): 利用するcommon.Messageを指定
        """

        self.MESSAGE = message

    # googleのReCaptcha画面からデータを抽出する
    def get_google_recaptcha_data(self, html: str):
        """get_google_recaptcha_data

        ReCapthcaのhtmlからsitekey, data-sの値を抽出する.


        Args:
            html (str): 解析するReCaptcha画面のhtmlデータ

        Returns:
            sitekey (str): 2Captchaへ送るsitekey
            data-s (str): 2Captchaへ送るdata-s
        """

        # resultの初期値設定
        sitekey = None
        data_s = None

        # ReCaptchaのタグ・要素データを宣言
        recaptcha_tag = '#captcha-form > #recaptcha'
        sitekey_el_name = 'data-sitekey'
        data_s_el_name = 'data-s'

        # htmlをBeautifulSoupで解析する
        soup = BeautifulSoup(html, 'lxml')

        # 要素を抽出する
        if recaptcha_tag != '':
            elements = soup.select(recaptcha_tag)

            # 要素のチェック
            if len(elements) > 0:
                el = elements[0]

                try:
                    sitekey = el[sitekey_el_name]
                    data_s = el[data_s_el_name]

                    return sitekey, data_s

                except AttributeError:
                    None

        return sitekey, data_s

    def in_php(self, data: dict):
        """in_php

        Args:
            data (dict): in.phpにpostするデータ(dict)

        Returns:
            bool: 処理が正常終了か否か
            str: request_code
        """

        res = self.session.post(self.api_in_url, data=data)

        if self.MESSAGE is not None:
            self.MESSAGE.print_text(
                '2Captcha Response in.php from `{}`: {}'.format(
                    self.api_in_url, res.text),
                mode='debug',
                header=self.MESSAGE.HEADER + ': ' + Color.GRAY +
                '[DEBUG]: [2CaptchaIn]' + Color.END,
                separator=": "
            )

        # status codeを確認
        if res.status_code == 200:
            d = json.loads(res.text)
            if d['status'] == 1:
                request_id = d['request']

                return True, request_id

        # request codeを取得できなかった場合、
        return False, None

    def res_php(self, request_id: str):
        """res_php

        Args:
            request_id (str): 2Captchaのres.php(2Captchaの突破状況確認するpath)で利用するrequest_id.

        Returns:
            (str): res.phpからのresponse結果を返す
        """

        url_param = {
            'key': self.api_key,
            'action': 'get',
            'json': 1,
            'id': request_id
        }
        params = parse.urlencode(url_param)
        target_url = self.api_res_url + '?' + params

        result = self.session.get(target_url)

        if self.MESSAGE is not None:
            self.MESSAGE.print_text(
                '2Captcha res.php Response from `{}`: {}'.format(
                    target_url, result.text),
                mode='debug',
                header=self.MESSAGE.HEADER + ': ' + Color.GRAY +
                '[DEBUG]: [2CaptchaRes]' + Color.END,
                separator=": "
            )

        return result

    # 解析結果を渡す
    def google_recaptcha(self, html: str, url: str, cookies: list, proxy: str):
        """[summary]

        Args:
            sitekey (str): ReCaptchaのhtml.
            url (str): ReCaptchaが表示されてしまったurl(元のurl)
            cookie (list): cookiesを渡す.
            proxy (str): proxyをuriで渡す.

        Returns:
            (str): Google ReCaptchaで使用するcodeを返す.
        """

        # code
        code = None
        result = None

        # set proxy
        self.proxy = proxy

        # sitekey, data-sを取得する
        sitekey, data_s = self.get_google_recaptcha_data(html)

        # proxyをuriから整形する
        proxy_parse = urlparse(proxy)
        proxy_type = proxy_parse.scheme.upper()
        proxy_uri = proxy_parse.netloc

        # cookieを整形する
        cookie_elements = []
        for cookie in cookies:
            cookie_element = cookie['name'] + ':' + cookie['value']
            cookie_elements.append(cookie_element)

        cookie_data = ';'.join(cookie_elements)

        # postリクエストで使用するデータを生成する
        payload = {
            'key': self.api_key,
            'pageurl': url,
            'method': 'userrecaptcha',
            'json': 1,
            'googlekey': sitekey,
            'data-s': data_s,
            'proxytype': proxy_type,
            'proxy': proxy_uri,
            'cookies': cookie_data,
            'callback': 'submitCallback',
        }

        if self.user_agent is not None:
            payload['userAgent'] = self.user_agent

        while True:
            # debug message
            if self.MESSAGE is not None:
                self.MESSAGE.print_text(
                    'Send ReCaptcha Data to `{}`.'.format(
                        self.api_in_url),
                    mode='info',
                    header=self.MESSAGE.HEADER + ': ' + Color.GRAY +
                    '[DEBUG]: [ReCaptcha]' + Color.END,
                    separator=": "
                )

            # リクエストを送信
            ok, request_id = self.in_php(payload)

            if not ok:
                # debug message
                if self.MESSAGE is not None:
                    self.MESSAGE.print_text(
                        'Failed Send ReCaptcha Data. data: {}'.format(
                            payload),
                        mode='warn',
                        header=self.MESSAGE.HEADER + ': ' + Color.GRAY +
                        '[DEBUG]: [ReCaptcha]' + Color.END,
                        separator=": "
                    )

                break

            # message
            if self.MESSAGE is not None:
                self.MESSAGE.print_text(
                    'Get request_id: {}'.format(request_id),
                    mode='info',
                    header=self.MESSAGE.HEADER + ': ' + Color.GRAY +
                    '[DEBUG]: [2Captcha]' + Color.END,
                    separator=": "
                )

                self.MESSAGE.print_text(
                    'Check ReCaptcha Rsponse Status from: {}'.format(
                        self.api_res_url),
                    mode='info',
                    header=self.MESSAGE.HEADER + ': ' + Color.GRAY +
                    '[DEBUG]: [2Captcha]' + Color.END,
                    separator=": "
                )

            # res_phpのチェックループ
            while True:
                res = self.res_php(request_id)

                # レスポンス(json)から読み込む
                data = json.loads(res.text)

                # codeを取得
                code = data['request']

                if data['status'] == 1:
                    result = code
                    return result

                if code != 'CAPCHA_NOT_READY':
                    break

                sleep(30)

        if code is None:
            code = 'None'

        # debug messages
        if self.MESSAGE is not None:
            self.MESSAGE.print_text(
                'Bypass NG ReCaptcha Data. code: {}'.format(code),
                mode='warn',
                header=self.MESSAGE.HEADER + ': ' + Color.GRAY +
                '[DEBUG]: [2Captcha]' + Color.END,
                separator=": "
            )

        return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================


import os
import platform

import setuptools

cmdclass = {}
try:
    from sphinx.setup_command import BuildDoc
    cmdclass = {'build_sphinx': BuildDoc}
except ImportError:
    pass

try:
    with open('README.rst') as f:
        readme = f.read()
except IOError:
    readme = ''


# 補完ファイルインストール用関数
def get_data_files():

    # 補完ファイルのインストール先を取得する関数
    def get_completefile_install_location(shell):
        # pathのprefixを定義
        prefix = ''

        # osの種類を取得
        uname = platform.uname()[0]

        # 実行ユーザがrootかどうかでprefixを変更
        if os.geteuid() == 0:
            ''' システムインストール時の挙動 '''
            if uname == 'Linux' and shell == 'bash':
                prefix = '/'
            elif uname == 'Linux' and shell == 'zsh':
                prefix = '/usr/local'
            elif uname == 'Darwin' and shell == 'bash':
                prefix = '/'
            elif uname == 'Darwin' and shell == 'zsh':
                prefix = '/usr'

        # shellの種類に応じてインストール先のlocationを変更
        if shell == 'bash':
            location = os.path.join(prefix, 'etc/bash_completion.d')
        elif shell == 'zsh':
            location = os.path.join(prefix, 'share/zsh/site-functions')
        else:
            raise ValueError('unsupported shell: {0}'.format(shell))

        # locationを返す
        return location

    # locationをdict形式で取得する
    loc = {
        'bash': get_completefile_install_location('bash'),
        'zsh': get_completefile_install_location('zsh')
    }

    # 対象となるファイルをdict形式で指定
    files = dict(
        bash=['completion/pydork-completion.bash'],
        zsh=[
            'completion/pydork-completion.bash',
            'completion/_pydork'
        ]
    )

    # data_files形式でreturn
    data_files = []
    data_files.append((loc['bash'], files['bash']))
    data_files.append((loc['zsh'], files['zsh']))
    return data_files


name = 'pydork'
version = '1.1.5'
release = '1.1.5'

if __name__ == "__main__":
    setuptools.setup(
        name=name,
        version=version,
        author='blacknon',
        author_email='blacknon@orebibou.com',
        maintainer='blacknon',
        maintainer_email='blacknon@orebibou.com',
        description='Scraping and listing text and image searches on Google, Bing, DuckDuckGo, Baidu, Yahoo japan.',
        long_description=readme,
        license='MIT License',
        install_requires=[
            'bs4',
            'get-chrome-driver',
            'get-gecko-driver',
            'chromedriver_autoinstaller',
            'geckodriver_autoinstaller',
            'fake_useragent',
            'lxml',
            'requests[socks]',
            'selenium==4.7.2',
            'selenium_requests',
            'pickle-mixin',
            'sphinx',
            'sphinx-rtd-theme',
            'sphinx-autobuild'
        ],
        url='https://github.com/blacknon/pydork',
        packages=setuptools.find_packages(),
        py_modules=['pydork'],
        entry_points={
            'console_scripts': [
                'pydork = pydork:main',
            ],
        },
        classifiers=[
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
            'License :: OSI Approved :: MIT License',
        ],
        data_files=get_data_files(),
        cmdclass=cmdclass,
        command_options={
            'build_sphinx': {
                'project': ('setup.py', name),
                'version': ('setup.py', version),
                'release': ('setup.py', release)}},
        setup_requires=[
            "sphinx",
            "sphinx-rtd-theme",
            "sphinx-autobuild",
        ],
    )
